@@ -59,130 +59,110 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: cardif_linux.c,v 1.24 2004/04/18 03:28:26 chessing Exp $
- * $Date: 2004/04/18 03:28:26 $
+ * $Id: cardif_linux.c,v 1.43 2004/08/20 04:42:28 chessing Exp $
+ * $Date: 2004/08/20 04:42:28 $
  * $Log: cardif_linux.c,v $
- * Revision 1.24  2004/04/18 03:28:26  chessing
+ * Revision 1.43  2004/08/20 04:42:28  chessing
+ * Set all of the new scanning and WPA code to be disabled by default, so that a 1.0.1 release can be made.  Added additional error message output to AKA code.  Fixed a few memory leaks in the AKA code.
  *
- * Fixed a little bit of verbage in cardif_linux.c that could be confusing.
+ * Revision 1.42  2004/08/19 04:11:48  chessing
+ * More updates to get closer to WPA support.  We can now associate to an AP that has WPA, however we are using a static WPA IE, so that needs to be fixed.  There is also still a fair bit of other things that need to be cleaned up, but we are closer.
  *
- * Revision 1.23  2004/04/01 06:12:55  npetroni
- * fixed off-by-one error
+ * Revision 1.41  2004/08/19 02:28:07  chessing
+ * First piece of WPA patch.  (The patch is growing fast, and this commit is to save what is done so far.)
  *
- * Revision 1.22  2004/03/29 21:36:38  chessing
+ * Revision 1.40  2004/08/16 23:36:28  chessing
+ * Cosmetic fixes and trapping of an error that popped up.
  *
- * Fixed a problem that would cause XSupplicant to segfault if there was no default network profile defined, an interface was down, and XSupplicant was terminated.  (All at the same time.)
+ * Revision 1.39  2004/08/16 01:56:44  chessing
  *
- * Revision 1.21  2004/03/27 01:40:45  chessing
+ * A few more updates to last nights patch.
  *
- * Lots of small updates to free memory that wasn't getting freed, add some additional debug output, and fix a couple of memory leaks.
+ * Revision 1.38  2004/08/15 04:29:35  chessing
  *
- * Revision 1.20  2004/03/26 21:34:52  chessing
- * Fixed problem with interface being down on startup causing xsupplicant to not read the proper configuration information when the interface is brought up.  Added/fixed code to rebuild userdata piece of structure when the essid changes.  Added code to avoid setting a key on an interface if the interface doesn't already have encryption enabled.  Added a little bit of debugging code to help find a solution to an IPC socket problem.
+ * Completed support for scanning and joining wireless networks.  If your interface isn't configured, we will scan to gather information about all known wireless networks.  If a network in the list has a configuration, we will join it.  (Including turning on needed encryption.)  There are still a few rough edges that need to be hammered out.  But the overall user experience should be improved.
  *
- * Revision 1.19  2004/03/25 06:06:57  chessing
+ * Revision 1.37  2004/08/12 01:40:06  chessing
  *
- * Some debug code cleanups.  Fixed a bug with non-existant, or down interfaces defined in the allow_interfaces would loop forever.  Added calls to reset wireless keys to all 0s when we end up in disconnected, or held state.
+ * Added "association" option to the global configuration file sections.  It can be set to either auto, or manual.  Setting "association" to auto means that XSupplicant will attempt to associate to a network with the lowest value in the "priority" setting.  The default for "association" is auto.  The other option added is "priority" which is a numeric value from 1..256.  A value of 1 is the highest priority, and will be the first network chosen.  (Assuming the network can be found in a scan.)  The default value for "priority" is 0, which means that network will be checked last.   --  Although the options are now in the config file, the actual functionality for auto association/priority is not yet complete.  (So, basically these options don't do anything yet. ;)
  *
- * Revision 1.18  2004/03/24 18:35:47  chessing
+ * Revision 1.36  2004/08/10 01:59:26  chessing
  *
- * Added a modified version of a patch from David Relson to fix a problem with some of the debug info in config_grammer.y.  Added some additional checks to eapol_key_type1 that will keep us from segfaulting under some *REALLY* strange conditions.  Changed the set key code in cardif_linux to double check that we aren't a wireless interface before returning an error.  This resolved a problem when XSupplicant was started when an interface was done.  Upon bringing up the interface, XSupplicant would sometimes think it wasn't wireless, and not bother trying to set keys.
+ * Added support for the SNMP supplicant counters defined in the IEEE 802.1X-2001 document.
  *
- * Revision 1.17  2004/03/23 23:34:20  galimorerpg
- * Removed another un-needed Makefile and added the cardif_get_int patch from Pavel Roskin
+ * Revision 1.35  2004/08/05 23:56:21  chessing
  *
- * Revision 1.16  2004/03/22 00:41:00  chessing
+ * Added basic support for scanning for broadcast SSIDs.  This is another step closer to WPA/11i support. ;)
  *
- * Added logfile option to the global config options in the config file.  The logfile is where output will go when we are running in daemon mode.  If no logfile is defined, output will go to the console that started xsupplicant.   Added forking to the code, so that when started, the process can daemonize, and run in the background.  If there is a desire to force running in the foreground (such as for debugging), the -f option was added.
+ * Revision 1.34  2004/07/26 03:08:58  chessing
  *
- * Revision 1.15  2004/03/06 03:53:54  chessing
+ * Fixed a few bugs in the new hot plug code.  Hot unplugging (and replugging) now works.
  *
- * We now send logoffs when the process is terminated.  Added a new option to the config file "wireless_control" which will allow a user to disable non-EAPoL key changes.  Added an update to destination BSSID checking that will reset the wireless key to all 0s when the BSSID changes.  (This is what "wireless_control" disables when it is set to no.)  Roaming should now work, but because we are resetting keys to 128 bit, there may be issues with APs that use 64 bit keys.  I will test this weekend.
+ * Revision 1.33  2004/07/25 19:36:41  chessing
+ * Fixed a few more logical/bitwise nots.  Added rtnetlink support for hot-plugging interfaces.  (Hot unplugging is coming soon. ;)
  *
- * Revision 1.14  2004/02/06 06:13:31  chessing
+ * Revision 1.32  2004/07/23 04:05:50  chessing
+ * Fixed a segfault problem.  Started to add rtnetlink support.
  *
- * Cleaned up some unneeded stuff in the configure.in file as per e-mail from Rakesh Patel.  Added all 12 patches from Jouni Malinen (Including wpa_supplicant patch, until we can add true wpa support in xsupplicant.)
+ * Revision 1.31  2004/07/22 22:35:07  chessing
  *
- * Revision 1.13  2004/01/20 03:44:32  chessing
+ * Changed cardif pieces to build as an intermediate module, to allow it to be defined in more than one file.  These changes should help make things more portable.
  *
- * A couple of small updates.  TTLS now uses the correct phase 2 type as defined by the config file.  Setting dest_mac now works, and has the desired results.  One small fix to EAP-SIM.
+ * Revision 1.30  2004/07/21 22:47:53  chessing
  *
- * Revision 1.12  2004/01/17 21:16:15  chessing
+ * Fixed bitwise not problem identified in bugid 995523.
  *
- * Various segfault fixes.  PEAP now works correctly again.  Some new error checking in the tls handlers.  Fixes for the way we determine if we have changed ESSIDs.  We now quit when we don't have a config, or when the config is bad. Added code to check and see if a frame is in the queue, and don't sleep if there is.  "Fixed" ID issue by inheriting the ID from the parent where needed.  However, assigning an ID inside of a handler will override the parent ID.  This could cause problems with some EAP types.  We should add a "username" field to PEAP to allow configuration of the inner EAP identity.
+ * Revision 1.29  2004/07/19 02:43:16  chessing
  *
- * Revision 1.11  2004/01/15 23:45:11  chessing
+ * Changed things to get rid of Linux specific pieces in the interface_data struct. Fixed up EAP-SIM and EAP-AKA to skip AVPs that we don't support.  (We print a mesage, and move on.)  Added the --enable-radiator-test option to configure EAP-AKA to use the test vectors included with Radiator's AKA module.  (To use this mode, no SIM card is required, but the PCSC library is still needed to build.  Also, some errors will be displayed.)
  *
- * Fixed a segfault when looking for wireless interfaces when all we had was a wired interface.  Fixed external command execution so that junk doesn't end up in the processed string anymore.  Changed the state machine to call txRspAuth even if there isn't a frame to process.  This will enable EAP methods to request information from a GUI interface (such as passwords, or supply challenge information that might be needed to generate passwords).  EAP methods now must decide what to do when they are handed NULL for the pointer to the in frame.  If they don't need any more data, they should quietly exit.
+ * Revision 1.28  2004/07/15 05:49:22  chessing
  *
- * Revision 1.10  2004/01/15 01:12:45  chessing
+ * We now reset ALLMULTI to the state is was in when we start XSupplicant.  Also, when we have to reset keys, we now reset all of them, instead of just the first one.
  *
- * Fixed a keying problem (keying material wasn't being generated correctly).  Added support for global counter variables from the config file. (Such as auth_period)  Added support for executing command defined in the config file based on different events.  (Things such as what to do on reauth.)  Added the ability to roam to a different SSID.  We now check to make sure our BSSID hasn't changed, and we follow it, if it has.  Fixed a sefault when the program was terminated in certain states.  Added attempt at better garbage collection on program termination. Various small code cleanups.
+ * Revision 1.27  2004/07/15 04:15:35  chessing
  *
- * Revision 1.9  2004/01/14 22:07:25  chessing
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Fixes that were needed in order to allow us to authenticate correctly.  We should now be able to authenticate using only information provided by the config file!
+ * Revision 1.26  2004/06/21 05:19:12  chessing
  *
- * Revision 1.8  2004/01/06 23:35:07  chessing
+ * Added a few minor fixes to EAP-AKA support.  Added "allmulti" as a global configuration option.  (By default, allmulti is now on!)
  *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.7  2003/12/28 07:13:21  chessing
- *
- * Fixed a problem where we would segfault on an EAP type we didn't understand.  Added EAP-OTP.  EAP-OTP has been tested using the opie package, and Radiator 3.8.  EAP-OTP currently prompts for a passphrase, which it shouldn't do, so it should be considered *VERY* much in test mode until we finish the GUI.
- *
- * Revision 1.6  2003/12/19 06:29:57  chessing
- *
- * New code to determine if an interface is wireless or not.  Lots of IPC updates.
- *
- * Revision 1.5  2003/12/04 04:36:25  chessing
- *
- * Added support for multiple interfaces (-D now works), also added DEBUG_EXCESSIVE to help clean up some of the debug output (-d 6).
- *
- * Revision 1.4  2003/11/29 03:50:03  chessing
- *
- * Added NAK code, EAP Type checking, split out daemon config from user config, added Display of EAP-Notification text, revamped phase 2 selection method for TTLS.
- *
- * Revision 1.3  2003/11/24 04:56:04  chessing
- *
- * EAP-SIM draft 11 now works.  Statemachine updated to work based on the up/down state of an interface, rather than just assuming it is up.
- *
- * Revision 1.2  2003/11/24 02:14:08  chessing
- *
- * Added EAP-SIM (draft 11 still needs work), various small changes to eap calls, new hex dump code including ASCII dump (used mostly for dumping frames)
- *
- * Revision 1.1.1.1  2003/11/19 04:13:28  chessing
- * New source tree
+ * Revision 1.25  2004/06/15 03:22:29  chessing
  *
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
 
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <fcntl.h>
-#include <linux/if.h>
-#include <linux/if_packet.h>
-#include <linux/if_ether.h>
 #include <string.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
 #include <linux/wireless.h>
+#include <linux/if_packet.h>
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
 #include <net/if_arp.h>
 
-#include "cardif/cardif.h"
-#include "config.h"
-#include "profile.h"
-#include "xsup_debug.h"
-#include "xsup_err.h"
+#include "../config.h"
+#include "../profile.h"
+#include "linux/cardif_linux_ssids.h"
+#include "cardif.h"
+#include "../xsup_debug.h"
+#include "../xsup_err.h"
+#include "linux/cardif_linux.h"
 
 #ifndef ETH_P_EAPOL
 #define ETH_P_EAPOL 0x888e
 #endif
 
+#define SCAN_TIMEOUT  30   // Wait 30 seconds before scanning again.
+
+// Define this, so the compiler doesn't complain.
+extern unsigned int if_nametoindex(const char *);
+
 /***********************************************
  *
  * Do whatever is needed to get the interface in to a state that we can send
@@ -193,15 +173,32 @@
 int cardif_init(struct interface_data *thisint)
 {
   struct ifreq ifr;
+  struct lin_sock_data *sockData;
   int sockopts, sockerr, retval;
-  //  char newdest[6];
 
   debug_printf(DEBUG_INT, "Initializing socket for interface %s..\n",
 	       thisint->intName);
 
+  // Find out what the interface index is.
+  thisint->intIndex = if_nametoindex(thisint->intName);
+  debug_printf(DEBUG_INT, "Index : %d\n", thisint->intIndex);
+
+  // Allocate memory for the things we need.
+  thisint->sockData = (void *)malloc(sizeof(struct lin_sock_data));
+  if (thisint->sockData == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Error allocating memory!\n");
+      return XEMALLOC;
+    }
+
+  sockData= thisint->sockData;
+
+  // Amount of time to wait before allowing a scan again.
+  sockData->scan_timer = 0;
+
   // Establish a socket handle.
-  thisint->sockInt = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_EAPOL));
-  if (thisint->sockInt < 0)
+  sockData->sockInt = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_EAPOL));
+  if (sockData->sockInt < 0)
     {
       debug_printf(DEBUG_NORMAL, 
 		   "Couldn't initialize raw socket for interface %s!\n",
@@ -212,7 +209,7 @@ int cardif_init(struct interface_data *thisint)
   // Tell the ifreq struct which interface we want to use.
   strncpy((char *)&ifr.ifr_name, thisint->intName, sizeof(ifr.ifr_name));
 
-  retval = ioctl(thisint->sockInt, SIOCGIFINDEX, &ifr);
+  retval = ioctl(sockData->sockInt, SIOCGIFINDEX, &ifr);
   if (retval < 0)
     {
       debug_printf(DEBUG_NORMAL, "Error getting interface index value for interface %s\n",
@@ -222,12 +219,12 @@ int cardif_init(struct interface_data *thisint)
 
   // Build our link layer socket struct, so we can bind it to a specific
   // interface.
-  thisint->sll.sll_family = PF_PACKET;
-  thisint->sll.sll_ifindex = ifr.ifr_ifindex;
-  thisint->sll.sll_protocol = htons(ETH_P_EAPOL);
+  sockData->sll.sll_family = PF_PACKET;
+  sockData->sll.sll_ifindex = ifr.ifr_ifindex;
+  sockData->sll.sll_protocol = htons(ETH_P_EAPOL);
 
   // Bind to the interface.
-  retval = bind(thisint->sockInt, (const struct sockaddr *)&thisint->sll, 
+  retval = bind(sockData->sockInt, (const struct sockaddr *)&sockData->sll, 
 		sizeof(struct sockaddr_ll));
   if (retval < 0)
     {
@@ -237,7 +234,7 @@ int cardif_init(struct interface_data *thisint)
     }
 
   // Get our MAC address.  (Needed for sending frames out correctly.)
-  retval = ioctl(thisint->sockInt, SIOCGIFHWADDR, &ifr);
+  retval = ioctl(sockData->sockInt, SIOCGIFHWADDR, &ifr);
   if (retval < 0)
     {
       debug_printf(DEBUG_NORMAL, "Error getting hardware (MAC) address for interface %s!\n",
@@ -248,8 +245,35 @@ int cardif_init(struct interface_data *thisint)
   // Store a copy of our source MAC for later use.
   memcpy((char *)&thisint->source_mac[0], (char *)&ifr.ifr_hwaddr.sa_data[0], 6);
 
+  // Check if we want ALLMULTI mode, and enable it.
+  if (config_get_allmulti() == 1)
+    {
+      // Tell the ifreq struct which interface we want to use.
+      strncpy((char *)&ifr.ifr_name, thisint->intName, sizeof(ifr.ifr_name));
+
+      if (ioctl(sockData->sockInt, SIOCGIFFLAGS, &ifr) < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't determine if ALLMULTI is enabled!\n");
+	} else {
+	  if (ifr.ifr_flags & IFF_ALLMULTI)
+	    {
+	      debug_printf(DEBUG_INT, "Allmulti mode is already enabled on this device!\n");
+	      thisint->flags |= ALLMULTI;
+	    } else {
+	      debug_printf(DEBUG_INT, "Allmulti is currently disabled on this device!\n");
+	      thisint->flags &= ~ALLMULTI;
+	    }
+	}
+
+      ifr.ifr_flags |= IFF_ALLMULTI;
+      if (ioctl(sockData->sockInt, SIOCSIFFLAGS, &ifr) < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't set ALLMULTI mode on this interface!  We will continue anyway!\n");
+	}
+    }
+
   // Set our socket to non-blocking.
-  sockopts = fcntl(thisint->sockInt, F_GETFL, 0);
+  sockopts = fcntl(sockData->sockInt, F_GETFL, 0);
   if (sockopts < 0)
     {
       debug_printf(DEBUG_NORMAL, "Error getting socket options for interface %s!\n",
@@ -257,7 +281,7 @@ int cardif_init(struct interface_data *thisint)
       return XENOTINT;
     }
 
-  sockerr = fcntl(thisint->sockInt, F_SETFL, sockopts | O_NONBLOCK);
+  sockerr = fcntl(sockData->sockInt, F_SETFL, sockopts | O_NONBLOCK);
   if (sockerr < 0)
     {
       debug_printf(DEBUG_NORMAL, "Error setting socket options for interface %s!\n",
@@ -270,6 +294,25 @@ int cardif_init(struct interface_data *thisint)
   return XENONE;
 }
 
+/****************************************************************
+ *
+ * This is called each time the internal clock ticks.  (Once a second.)
+ * It should be used to update any timers that are to be used inside
+ * of cardif.
+ *
+ ****************************************************************/
+void cardif_clock_tick(struct interface_data *intdata)
+{
+  struct lin_sock_data *sockData;
+
+  sockData= intdata->sockData;
+
+  if (sockData->scan_timer > 0)
+    {
+      sockData->scan_timer--;
+    }
+}
+
 /**************************************************************
  * 
  * Check if encryption is available.  If it is, we will return
@@ -312,6 +355,42 @@ int cardif_enc_enabled(struct interface_data *thisint)
   return rc;
 }
 
+/**************************************************************
+ *
+ * Force the card in to open mode.  (The call cardif_enc_open will only
+ * set the card to open mode if it has been in the past.)
+ *
+ **************************************************************/
+int cardif_force_enc_on_open(struct interface_data *thisint)
+{
+  int rc = 0;
+  char key[5]={0x00,0x00,0x00,0x00,0x00};
+  struct iwreq wrq;
+  struct lin_sock_data *sockData;
+
+  if (!(thisint->flags & WEP))
+    {
+      debug_printf(DEBUG_INT, "WEP not desired!\n");
+      return XENONE;
+    }
+
+  sockData = thisint->sockData;
+  bzero((struct iwreq *)&wrq, sizeof(struct iwreq));
+
+  strncpy(wrq.ifr_name, thisint->intName, IFNAMSIZ);
+  wrq.u.data.pointer = (caddr_t)&key;
+  wrq.u.data.flags = 0;
+  wrq.u.data.length = 5;
+
+  if (ioctl(sockData->sockInt, SIOCSIWENCODE, &wrq) < 0)
+    {
+      debug_printf(DEBUG_INT, "Error setting open encoding mode!\n");
+      rc = -1;
+    }
+ 
+  return rc;
+}
+
 /**************************************************************
  *
  * If we have detected, or forced this interface to reset keys, then
@@ -323,6 +402,9 @@ void cardif_reset_keys(struct interface_data *thisint)
 {
   char zerokey[13];
   char keylen = 13;
+#ifdef EXPERIMENTAL
+  struct ssids_list *ourssid;
+#endif
 
   if (thisint->userdata == NULL) 
     {
@@ -336,17 +418,147 @@ void cardif_reset_keys(struct interface_data *thisint)
       return;
     }
 
-  if (cardif_enc_enabled(thisint) != TRUE)
+#ifdef EXPERIMENTAL
+  ourssid = cardif_linux_ssids_find(thisint);
+
+  if (ourssid == NULL)
+    {
+      // Leave this as a debug INT because it is more informational, and in 
+      // the event we have an invalid SSID for a long time the log will fill
+      // up with these messages...  Not pretty!
+      debug_printf(DEBUG_INT, "Couldn't find any information about the SSID we are using!\n");
+      return;
+    }
+
+  if (!(ourssid->flags & WEP)) 
+    {
+      debug_printf(DEBUG_INT, "SSID doesn't appear to use encryption!  Ignoring!\n");
+      return;
+    }
+#else
+    if (cardif_enc_enabled(thisint) != TRUE)
     {
       debug_printf(DEBUG_INT, "Encryption appears to be disabled.  We will not reset keys on interface %s!\n", thisint->intName);
       return;
     }
-  
+#endif
+
   bzero(&zerokey, 13);
 
   // We set the key index to 0x80, to force key 0 to be set to all 0s,
   // and to have key 0 be set as the default transmit key.
   set_wireless_key(thisint, (char *)&zerokey, keylen, 0x80);
+  set_wireless_key(thisint, (char *)&zerokey, keylen, 0x01);
+  set_wireless_key(thisint, (char *)&zerokey, keylen, 0x02);
+  set_wireless_key(thisint, (char *)&zerokey, keylen, 0x03);
+}
+
+/**************************************************************
+ *
+ * Tell the wireless card to start scanning for wireless networks.
+ *
+ **************************************************************/
+int cardif_start_wireless_scan(struct interface_data *thisint)
+{
+#ifdef EXPERIMENTAL
+  struct lin_sock_data *sockData;
+  struct iwreq iwr;
+
+  if (!(thisint->flags & IS_WIRELESS))
+    {
+      debug_printf(DEBUG_INT, "%s is not a wireless interface!\n", 
+		   thisint->intName);
+      return XENOWIRELESS;
+    }
+
+  if (thisint->flags & SCANNING)
+    {
+      debug_printf(DEBUG_INT, "Already scanning!\n");
+      return XENONE;
+    }
+
+  sockData = thisint->sockData;
+
+  if (sockData->scan_timer > 0)
+    {
+      debug_printf(DEBUG_INT, "Scan timer has not expired!  Not scanning!\n");
+      return XENONE;
+    }
+
+  debug_printf(DEBUG_INT, "Issuing scan request for interface %s!\n",
+	       thisint->intName);
+
+  // Clear out our active list of SSIDs.
+  cardif_linux_ssids_clear(thisint);
+
+  iwr.u.param.flags = IW_SCAN_DEFAULT;
+  iwr.u.param.value = 0;
+  strcpy((char *)&iwr.ifr_name, thisint->intName);
+
+  if (ioctl(sockData->sockInt, SIOCSIWSCAN, &iwr) < 0)
+    {
+      debug_printf(DEBUG_NORMAL, "Error with SCAN ioctl!  (Perhaps your card doesn't support scanning?)\n");
+      return -1;
+    }
+  debug_printf(DEBUG_EVERYTHING, "Flags before setting scan flag : %02X\n",
+	       thisint->flags);
+
+  SET_FLAG(thisint->flags, SCANNING);  // We are scanning.
+
+  debug_printf(DEBUG_EVERYTHING, "Flags after setting scan flag : %02X\n",
+	       thisint->flags);
+
+  sockData->scan_timer = SCAN_TIMEOUT;
+#endif
+  return XENONE;
+}
+
+/**************************************************************
+ *
+ * Check to see if the BSSID value is valid.  If it is, return TRUE. If
+ * it isn't return FALSE.
+ *
+ **************************************************************/
+int cardif_valid_dest(struct interface_data *thisint)
+{
+  char baddest[6], newdest[6];
+
+  if ((thisint->flags & IS_WIRELESS) && 
+      (GetBSSID(thisint, (char *)&newdest) == XENONE))
+    {
+      memset((char *)&baddest, 0x00, 6);
+      if (memcmp(thisint->dest_mac, baddest, 6) == 0)
+	{
+	  debug_printf(DEBUG_INT, "All 0s for dest mac!\n");
+#ifdef EXPERIMENTAL
+	  cardif_start_wireless_scan(thisint);
+#endif
+	  return FALSE;
+
+	}
+
+      memset((char *)&baddest, 0x44, 6);
+      if (memcmp(thisint->dest_mac, baddest, 6) == 0)
+	{
+	  debug_printf(DEBUG_INT, "All 4s for dest mac!\n");
+#ifdef EXPERIMENTAL
+	  cardif_start_wireless_scan(thisint);
+#endif
+	  return FALSE;
+	}
+
+      memset((char *)&baddest, 0xff, 6);
+      if (memcmp(thisint->dest_mac, baddest, 6) == 0)
+	{
+	  debug_printf(DEBUG_INT, "All Fs for dest mac!\n");
+#ifdef EXPERIMENTAL
+	  cardif_start_wireless_scan(thisint);
+#endif
+	  return FALSE;
+	}
+    }
+  
+  return TRUE;
 }
 
 /**************************************************************
@@ -361,13 +573,12 @@ void cardif_reset_keys(struct interface_data *thisint)
 int cardif_check_dest(struct interface_data *thisint)
 {
   char newdest[6], *newssid;
-  char baddest[6];
   int changed = FALSE;
 
   bzero((char *)&newdest, 6);
 
   // If we are on wireless, figure out the target MAC address.
-  if ((thisint->isWireless == TRUE) && 
+  if ((thisint->flags & IS_WIRELESS) && 
       (GetBSSID(thisint, (char *)&newdest) == XENONE))
     {
       if (memcmp(thisint->dest_mac, newdest, 6) != 0)
@@ -384,25 +595,15 @@ int cardif_check_dest(struct interface_data *thisint)
 	  cardif_reset_keys(thisint);
 	}
 
-      memset((char *)&baddest, 0x00, 6);
-      if (memcmp(thisint->dest_mac, baddest, 6) == 0)
+      if (cardif_valid_dest(thisint) == FALSE)
 	{
-	  debug_printf(DEBUG_INT, "We don't appear to be associated!  Resetting keys!\n");
 	  cardif_reset_keys(thisint);
-	}
+	  if ((!(thisint->flags & SCANNING)) && 
+	      (!(thisint->flags & SCAN_DONE))) 
+	    cardif_start_wireless_scan(thisint);
 
-      memset((char *)&baddest, 0x44, 6);
-      if (memcmp(thisint->dest_mac, baddest, 6) == 0)
-	{
-	  debug_printf(DEBUG_INT, "All 4s for dest mac! Resetting keys!\n");
-	  cardif_reset_keys(thisint);
-	}
-
-      memset((char *)&baddest, 0xff, 6);
-      if (memcmp(thisint->dest_mac, baddest, 6) == 0)
-	{
-	  debug_printf(DEBUG_INT, "All Fs for dest mac!  Resetting keys!\n");
-	  cardif_reset_keys(thisint);
+	  if (thisint->flags & SCAN_DONE)
+	    cardif_linux_ssids_join_network(thisint);
 	}
 
       // If we were able to get a BSSID, we should also try to get an SSID.
@@ -429,13 +630,25 @@ int cardif_check_dest(struct interface_data *thisint)
 	      newssid = NULL;
 	    }
 	}
-    } else {
-      //      debug_printf(DEBUG_INT, "Interface doesn't appear to be a wireless interface!\n");
-    }
+    } 
 
   return changed;
 }
 
+/******************************************
+ *
+ * Return the socket number for functions that need it.
+ *
+ ******************************************/
+int cardif_get_socket(struct interface_data *thisint)
+{
+  struct lin_sock_data *sockData;
+
+  sockData = thisint->sockData;
+
+  return sockData->sockInt;
+}
+
 /******************************************
  *
  * Clean up anything that was created during the initialization and operation
@@ -444,8 +657,51 @@ int cardif_check_dest(struct interface_data *thisint)
  ******************************************/
 int cardif_deinit(struct interface_data *thisint)
 {
+  struct ifreq ifr;
+  uint16_t int16;
+  struct lin_sock_data *sockData;
+
+  sockData = thisint->sockData;
+
   debug_printf(DEBUG_EVERYTHING, "Cleaning up interface %s...\n",thisint->intName);
-  close(thisint->sockInt);
+
+  // Check if we want ALLMULTI mode, and enable it.
+  if (config_get_allmulti() == 1)
+    {
+      // Tell the ifreq struct which interface we want to use.
+      strncpy((char *)&ifr.ifr_name, thisint->intName, sizeof(ifr.ifr_name));
+
+      if (ioctl(sockData->sockInt, SIOCGIFFLAGS, &ifr) < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't get interface flags!\n");
+	} else {
+	  // Check if allmulti was disabled when we started.  If it was,
+	  // then disable it again, so everything is good.
+	  if (!(thisint->flags & ALLMULTI))
+	    {
+	      debug_printf(DEBUG_INT, "Turning off ALLMULTI mode!\n");
+
+	      int16 = ifr.ifr_flags;
+
+	      // ANDing the flags with 0xfdff will turn off the ALLMULTI flag.
+	      ifr.ifr_flags = (int16 & 0xfdff);
+	      if (ioctl(sockData->sockInt, SIOCSIFFLAGS, &ifr) < 0)
+		{
+		  debug_printf(DEBUG_NORMAL, "Couldn't set ALLMULTI mode on this interface!  We will continue anyway!\n");
+		}
+	    }
+	}
+    }
+
+  close(sockData->sockInt);
+
+  // Now clean up the memory.
+  if (thisint->sockData != NULL)
+    {
+      free(thisint->sockData);
+      thisint->sockData = NULL;
+    }
+
   return XENONE;
 }
 
@@ -462,7 +718,7 @@ int set_wireless_key(struct interface_data *thisint, u_char *key, int keylen,
   int skfd;
   struct iwreq wrq;
 
-  if (thisint->isWireless == FALSE)
+  if (!(thisint->flags & IS_WIRELESS))
     {
       if ((cardif_int_is_wireless(thisint->intName) != TRUE) ||
 	  (thisint->userdata->type == WIRED) ||
@@ -471,7 +727,7 @@ int set_wireless_key(struct interface_data *thisint, u_char *key, int keylen,
 	  debug_printf(DEBUG_NORMAL, "Interface isn't wireless, but an attempt to set a key was made!\n");
 	  return XENOWIRELESS;
 	} else {
-	  thisint->isWireless = TRUE;
+	  thisint->flags |= IS_WIRELESS;
 	}
     }
 
@@ -502,7 +758,7 @@ int set_wireless_key(struct interface_data *thisint, u_char *key, int keylen,
 	  // This is a unicast key, use it for transmissions.
 	  strncpy(wrq.ifr_name, thisint->intName, IFNAMSIZ);
 
-	  wrq.u.data.flags = ((index & 0x7f) + 1) & IW_ENCODE_INDEX;
+	  wrq.u.data.flags = (((index & 0x7f) + 1) & IW_ENCODE_INDEX) | IW_ENCODE_NOKEY;
 	  wrq.u.data.flags |= IW_ENCODE_OPEN;
 
 	  wrq.u.data.length = 0;
@@ -522,6 +778,118 @@ int set_wireless_key(struct interface_data *thisint, u_char *key, int keylen,
   return rc;
 }
 
+/**********************************************************
+ *
+ * Set the SSID of the wireless card.
+ *
+ **********************************************************/
+int SetSSID(struct interface_data *thisint, char *ssid_name)
+{
+  struct iwreq iwr;
+  struct lin_sock_data *sockData;
+
+  if (thisint == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid interface struct passed in to %s!\n",
+		   __FUNCTION__);
+      return XEGENERROR;
+    }
+
+  sockData = thisint->sockData;
+ 
+  if (!(thisint->flags & IS_WIRELESS))
+    {
+      // We want to verify that the interface is in fact, not wireless, and
+      // not that we are in a situation where the interface has just been 
+      // down.
+      if (!(thisint->flags & WAS_DOWN))
+	{
+	  return XENOWIRELESS;
+	}
+    } 
+
+  // If we get here, and isWireless == FALSE, then we need to double
+  // check that our interface is really not wireless.
+  if (!(thisint->flags & IS_WIRELESS))
+    {
+      if (cardif_int_is_wireless(thisint->intName) == TRUE)
+	{
+	  thisint->flags |= IS_WIRELESS;
+	} else {
+	  thisint->flags &= (~IS_WIRELESS);
+	}
+
+      if (!(thisint->flags & IS_WIRELESS))
+	{
+	  thisint->flags &= (~WAS_DOWN);
+	}
+    }
+
+  // Specify the interface name we are asking about.
+  strncpy(iwr.ifr_name, thisint->intName, sizeof(iwr.ifr_name));
+
+  iwr.u.essid.pointer = (caddr_t) ssid_name;
+  iwr.u.essid.length = strlen(ssid_name);
+  iwr.u.essid.flags = 1;
+
+  if (ioctl(sockData->sockInt, SIOCSIWESSID, &iwr) < 0) return XENOWIRELESS;
+
+  thisint->flags &= (~WAS_DOWN);
+
+  return XENONE;
+}
+
+/******************************************
+ *
+ * Do whatever we need to do in order to associate based on the flags in
+ * the ssids_list struct.
+ *
+ ******************************************/
+void cardif_associate(struct interface_data *intdata, void *newssid)
+{
+#ifdef EXPERIMENTAL
+  struct ssids_list *ssid;
+
+  ssid = (struct ssids_list *)newssid;
+
+  if (intdata == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid interface struct passed to %s!\n",
+		   __FUNCTION__);
+      return;
+    }
+
+  if (ssid == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid SSID information passed to %s!\n",
+		   __FUNCTION__);
+      return;
+    }
+
+  debug_printf(DEBUG_INT, "Setting SSID to %s\n", ssid->ssidName);
+
+  if (ssid->flags & WEP)
+    {
+      debug_printf(DEBUG_INT, "Setting open encryption.\n");
+      cardif_force_enc_on_open(intdata);
+    }
+
+  if (ssid->flags & WPA_IE)
+    {
+      debug_printf(DEBUG_INT, "Enabling WPA!\n");
+      cardif_linux_wpa_set_wpa_ie(intdata, NULL, 0);
+      cardif_linux_wpa_enable(intdata);
+    }
+
+  // Otherwise, set the SSID.
+  if (SetSSID(intdata, ssid->ssidName) != XENONE)
+    {
+      debug_printf(DEBUG_NORMAL, "Couldn't associate to %s!\n", ssid->ssidName);
+      return;
+    }
+#endif
+}
+
 /******************************************
  *
  * Ask the wireless card for the ESSID that we are currently connected to.  If
@@ -532,13 +900,16 @@ int set_wireless_key(struct interface_data *thisint, u_char *key, int keylen,
 int GetSSID(struct interface_data *thisint, char *ssid_name)
 {
   struct iwreq iwr;
+  struct lin_sock_data *sockData;
+
+  sockData = thisint->sockData;
  
-  if (thisint->isWireless == FALSE)
+  if (!(thisint->flags & IS_WIRELESS))
     {
       // We want to verify that the interface is in fact, not wireless, and
       // not that we are in a situation where the interface has just been 
       // down.
-      if (thisint->wasDown == FALSE)
+      if (!(thisint->flags & WAS_DOWN))
 	{
 	  return XENOWIRELESS;
 	}
@@ -546,12 +917,18 @@ int GetSSID(struct interface_data *thisint, char *ssid_name)
 
   // If we get here, and isWireless == FALSE, then we need to double
   // check that our interface is really not wireless.
-  if (thisint->isWireless == FALSE)
+  if (!(thisint->flags & IS_WIRELESS))
     {
-      thisint->isWireless = cardif_int_is_wireless(thisint->intName);
-      if (thisint->isWireless == FALSE)
+      if (cardif_int_is_wireless(thisint->intName) == TRUE)
 	{
-	  thisint->wasDown = FALSE;
+	  thisint->flags |= IS_WIRELESS;
+	} else {
+	  thisint->flags &= (~IS_WIRELESS);
+	}
+
+      if (!(thisint->flags & IS_WIRELESS))
+	{
+	  thisint->flags &= (~WAS_DOWN);
 	}
     }
 
@@ -562,9 +939,9 @@ int GetSSID(struct interface_data *thisint, char *ssid_name)
   iwr.u.essid.length = 100;
   iwr.u.essid.flags = 0;
 
-  if (ioctl(thisint->sockInt, SIOCGIWESSID, &iwr) < 0) return XENOWIRELESS;
+  if (ioctl(sockData->sockInt, SIOCGIWESSID, &iwr) < 0) return XENOWIRELESS;
 
-  thisint->wasDown = FALSE;
+  thisint->flags &= (~WAS_DOWN);
 
   return XENONE;
 }
@@ -613,7 +990,8 @@ int cardif_check_ssid(struct interface_data *thisint)
 
 	  // Since we changed essids, we no longer have completed a
 	  // "first auth"
-	  thisint->firstauth = FALSE;
+	  thisint->flags &= (~FIRSTAUTH);
+	  //	  thisint->firstauth = FALSE;
 
 	  return XNEWESSID;
 	}
@@ -631,11 +1009,14 @@ int cardif_check_ssid(struct interface_data *thisint)
 int GetBSSID(struct interface_data *thisint, char *bssid_dest)
 {
   struct iwreq iwr;
+  struct lin_sock_data *sockData;
+
+  sockData = thisint->sockData;
 
   // Specify the interface name we are asking about.
   strncpy(iwr.ifr_name, thisint->intName, sizeof(iwr.ifr_name));
   
-  if (ioctl(thisint->sockInt, SIOCGIWAP, &iwr) < 0) return XENOWIRELESS;
+  if (ioctl(sockData->sockInt, SIOCGIWAP, &iwr) < 0) return XENOWIRELESS;
 
   memcpy(bssid_dest, iwr.u.ap_addr.sa_data, 6);
   return XENONE;
@@ -651,9 +1032,12 @@ int get_if_state(struct interface_data *thisint)
 {
   int retVal;
   struct ifreq ifr;
+  struct lin_sock_data *sockData;
+
+  sockData = thisint->sockData;
 
   strncpy(ifr.ifr_name, thisint->intName, sizeof(ifr.ifr_name));
-  retVal = ioctl(thisint->sockInt, SIOCGIFFLAGS, &ifr);
+  retVal = ioctl(sockData->sockInt, SIOCGIFFLAGS, &ifr);
   if (retVal < 0)
     {
       debug_printf(DEBUG_NORMAL, "Interface %s not found!\n", thisint->intName);
@@ -664,7 +1048,7 @@ int get_if_state(struct interface_data *thisint)
     {
       return TRUE;
     } else {
-      thisint->wasDown = TRUE;
+      thisint->flags |= WAS_DOWN;
       return FALSE;
     }
   return XENONE;
@@ -681,6 +1065,9 @@ int sendframe(struct interface_data *thisint, char *sendframe, int sendsize)
 {
   char nomac[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
   int retval;
+  struct lin_sock_data *sockData;
+
+  sockData = thisint->sockData;
 
   if (thisint == NULL) return XEMALLOC;
 
@@ -706,8 +1093,8 @@ int sendframe(struct interface_data *thisint, char *sendframe, int sendsize)
   debug_printf(DEBUG_EVERYTHING, "Frame to be sent : \n");
   debug_hex_dump(DEBUG_EVERYTHING, sendframe, sendsize);
 
-  retval = sendto(thisint->sockInt, sendframe, sendsize, 0,
-		  (struct sockaddr *)&thisint->sll, sizeof(thisint->sll));
+  retval = sendto(sockData->sockInt, sendframe, sendsize, 0,
+		  (struct sockaddr *)&sockData->sll, sizeof(sockData->sll));
   if (retval <= 0)
     {
       debug_printf(DEBUG_NORMAL, "Couldn't send frame! (%s)\n", strerror(errno));
@@ -727,11 +1114,14 @@ int getframe(struct interface_data *thisint, char *resultframe, int *resultsize)
 {
   int newsize=0;
   char dot1x_default_dest[6] = {0x01, 0x80, 0xc2, 0x00, 0x00, 0x03};
+  struct lin_sock_data *sockData;
+
+  sockData = thisint->sockData;
 
   errno = 0;
   *resultsize = 1550;
 
-  newsize = recvfrom(thisint->sockInt, resultframe, *resultsize, 0, 0, 0);
+  newsize = recvfrom(sockData->sockInt, resultframe, *resultsize, 0, 0, 0);
   if (newsize <= 0)
     {
       debug_printf(DEBUG_EXCESSIVE, "Couldn't get frame.  (Maybe there weren't any!)\n");
@@ -797,8 +1187,11 @@ int frameavail(struct interface_data *thisint)
 {
   int newsize=0;
   char resultframe[1520];
+  struct lin_sock_data *sockData;
 
-  newsize = recvfrom(thisint->sockInt, &resultframe, 1520, MSG_PEEK, 0, 0);
+  sockData = thisint->sockData;
+
+  newsize = recvfrom(sockData->sockInt, &resultframe, 1520, MSG_PEEK, 0, 0);
   if (newsize > 0) return TRUE;
 
   return FALSE;
@@ -943,8 +1336,3 @@ int cardif_int_is_wireless(char *interface)
     }
   return XENONE;   // No errors.
 }
-
-
-
-  
-
