@@ -59,174 +59,46 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: config.c,v 1.32 2004/03/26 21:34:51 chessing Exp $
- * $Date: 2004/03/26 21:34:51 $
+ * $Id: config.c,v 1.42 2004/08/19 02:28:06 chessing Exp $
+ * $Date: 2004/08/19 02:28:06 $
  * $Log: config.c,v $
- * Revision 1.32  2004/03/26 21:34:51  chessing
- * Fixed problem with interface being down on startup causing xsupplicant to not read the proper configuration information when the interface is brought up.  Added/fixed code to rebuild userdata piece of structure when the essid changes.  Added code to avoid setting a key on an interface if the interface doesn't already have encryption enabled.  Added a little bit of debugging code to help find a solution to an IPC socket problem.
+ * Revision 1.42  2004/08/19 02:28:06  chessing
+ * First piece of WPA patch.  (The patch is growing fast, and this commit is to save what is done so far.)
  *
- * Revision 1.31  2004/03/26 03:52:46  chessing
+ * Revision 1.41  2004/08/15 04:29:34  chessing
  *
- * Fixed a bug in xsup_debug that would cause config-parse to crash.  Added new key word for session resumption.  Added code to attempt session resumption.  So far, testing has not succeeded, but it is attempting resume. (Four TTLS packets are exchanged, and then we get a failure.)  More testing is needed.
+ * Completed support for scanning and joining wireless networks.  If your interface isn't configured, we will scan to gather information about all known wireless networks.  If a network in the list has a configuration, we will join it.  (Including turning on needed encryption.)  There are still a few rough edges that need to be hammered out.  But the overall user experience should be improved.
  *
- * Revision 1.30  2004/03/24 07:42:33  galimorerpg
- * Fixed a *NASTY* recursive loop in config.c/config_get_logfile()
- * Where config_get_logfile was debug_printf()ing and debug_printf()
- * was calling config_get_logfile.  I've worked around this by *NOT*
- * debug_printf()ing from config_get_logfile()
+ * Revision 1.40  2004/08/12 01:40:06  chessing
  *
- * On a side note, I found this bug because xsupplicant was crashing.
- * There's a handy option in the 2.4.25 kernel (CONFIG_OOM_KILLER) that
- * can kill programs that it decides are trying to use too much memory.
+ * Added "association" option to the global configuration file sections.  It can be set to either auto, or manual.  Setting "association" to auto means that XSupplicant will attempt to associate to a network with the lowest value in the "priority" setting.  The default for "association" is auto.  The other option added is "priority" which is a numeric value from 1..256.  A value of 1 is the highest priority, and will be the first network chosen.  (Assuming the network can be found in a scan.)  The default value for "priority" is 0, which means that network will be checked last.   --  Although the options are now in the config file, the actual functionality for auto association/priority is not yet complete.  (So, basically these options don't do anything yet. ;)
  *
- * Interestingly enough, valgrind didn't crash on this problem, presumably
- * because of its memory management?  Either that or the kernel didn't think
- * valgrind was hitting the memory hard enough.
+ * Revision 1.39  2004/07/25 19:36:40  chessing
+ * Fixed a few more logical/bitwise nots.  Added rtnetlink support for hot-plugging interfaces.  (Hot unplugging is coming soon. ;)
  *
- * Revision 1.29  2004/03/22 00:41:00  chessing
+ * Revision 1.38  2004/07/19 02:43:16  chessing
  *
- * Added logfile option to the global config options in the config file.  The logfile is where output will go when we are running in daemon mode.  If no logfile is defined, output will go to the console that started xsupplicant.   Added forking to the code, so that when started, the process can daemonize, and run in the background.  If there is a desire to force running in the foreground (such as for debugging), the -f option was added.
+ * Changed things to get rid of Linux specific pieces in the interface_data struct. Fixed up EAP-SIM and EAP-AKA to skip AVPs that we don't support.  (We print a mesage, and move on.)  Added the --enable-radiator-test option to configure EAP-AKA to use the test vectors included with Radiator's AKA module.  (To use this mode, no SIM card is required, but the PCSC library is still needed to build.  Also, some errors will be displayed.)
  *
- * Revision 1.28  2004/03/15 16:23:24  chessing
+ * Revision 1.37  2004/06/29 01:35:34  chessing
  *
- * Added some checks to TLS using EAP types to make sure the root certificate isn't set to NULL.  (If it is, we can't authenticate, so we bail out.)  Changed the user certificate settings in the config file to all start with user_.  So, "cert" is now "user_cert", "key" is now "user_key", and "key_pass" is now "user_key_pass".  The structures and other related variables were also updated to reflect this change.  THIS WILL PROBABLY BREAK CONFIG FILES FOR SOME USERS!  (Be prepared for complaints on the list!)  ;)
+ * Added credit to eapleap.c for Gilbert Goodwill (who gave me the clues I needed to figure out the keying piece for LEAP).  Added patch from Toby Collett to make PEAP work for broken implementations of IAS.  (New option ias_quirk has been added.)
  *
- * Revision 1.27  2004/03/06 03:53:52  chessing
+ * Revision 1.36  2004/06/22 20:14:22  chessing
  *
- * We now send logoffs when the process is terminated.  Added a new option to the config file "wireless_control" which will allow a user to disable non-EAPoL key changes.  Added an update to destination BSSID checking that will reset the wireless key to all 0s when the BSSID changes.  (This is what "wireless_control" disables when it is set to no.)  Roaming should now work, but because we are resetting keys to 128 bit, there may be issues with APs that use 64 bit keys.  I will test this weekend.
+ * Fixed some compile problems where SIM/AKA code was trying to compile even when it wasn't supposed to.
  *
- * Revision 1.26  2004/03/05 23:58:45  chessing
+ * Revision 1.35  2004/06/21 05:19:12  chessing
  *
- * Added CN (sometimes called server name) checking to TTLS and PEAP.  This resulted in two new config options in the eap-ttls, and eap-peap blocks.  cncheck should be the name (or partial name) to match in the CN.  cnexact should be yes/no depending on if we want to match the CN exactly, or just see if our substring is in the CN.
+ * Added a few minor fixes to EAP-AKA support.  Added "allmulti" as a global configuration option.  (By default, allmulti is now on!)
  *
- * Revision 1.25  2004/02/16 05:05:59  chessing
+ * Revision 1.34  2004/06/15 03:35:19  chessing
  *
- * Added support for the deny_interfaces, and allow_interfaces options in the config file.  (We should now have support for *EVERYTHING* in the new config file format!)  Updated EAP types other than SIM to use the new password prompt/delay code.  Phase 2 of TTLS still needs to be completed, along with the code that actually notifies the GUI.
+ * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
  *
- * Revision 1.24  2004/02/10 03:40:22  npetroni
- * updated config to include a phase 2 identity for PEAP
- *
- * Revision 1.23  2004/02/06 06:13:31  chessing
- *
- * Cleaned up some unneeded stuff in the configure.in file as per e-mail from Rakesh Patel.  Added all 12 patches from Jouni Malinen (Including wpa_supplicant patch, until we can add true wpa support in xsupplicant.)
- *
- * Revision 1.22  2004/01/15 23:45:10  chessing
- *
- * Fixed a segfault when looking for wireless interfaces when all we had was a wired interface.  Fixed external command execution so that junk doesn't end up in the processed string anymore.  Changed the state machine to call txRspAuth even if there isn't a frame to process.  This will enable EAP methods to request information from a GUI interface (such as passwords, or supply challenge information that might be needed to generate passwords).  EAP methods now must decide what to do when they are handed NULL for the pointer to the in frame.  If they don't need any more data, they should quietly exit.
- *
- * Revision 1.21  2004/01/15 01:12:44  chessing
- *
- * Fixed a keying problem (keying material wasn't being generated correctly).  Added support for global counter variables from the config file. (Such as auth_period)  Added support for executing command defined in the config file based on different events.  (Things such as what to do on reauth.)  Added the ability to roam to a different SSID.  We now check to make sure our BSSID hasn't changed, and we follow it, if it has.  Fixed a sefault when the program was terminated in certain states.  Added attempt at better garbage collection on program termination. Various small code cleanups.
- *
- * Revision 1.20  2004/01/14 05:44:48  chessing
- *
- * Added pid file support. (Very basic for now, needs to be improved a little.)  Attempted to add setup of global variables. (Need to figure out why it is segfaulting.)  Added more groundwork for IPC.
- *
- * Revision 1.19  2004/01/13 01:55:55  chessing
- *
- * Major changes to EAP related code.  We no longer pass in an interface_data struct to EAP handlers.  Instead, we hand in a generic_eap_data struct which containsnon-interface specific information.  This will allow EAP types to be reused as phase 2 type easier.  However, this new code may create issues with EAP types that make use of the identity in the eap type.  Somehow, the identity value needs to propigate down to the EAP method.  It currently does not.  This should be any easy fix, but more testing will be needed.
- *
- * Revision 1.18  2004/01/06 23:35:06  chessing
- *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.17  2004/01/06 22:25:58  npetroni
- * added crl parameter to tls, ttls, and peap and user cert,key,key_pass to ttls,peap
- *
- * Revision 1.16  2003/12/31 16:16:35  npetroni
- * made some generalizations to the way config code works so that now
- * it is easy to let any method be put inside of PEAP with little effort.
- *
- * Added MD5, SIM to the PEAP config section.
- *
- * Added allow types for OTP and GTC- we still need configuration parameters
- *   for these methods though.
- *
- * this code is coming together I think.
- *
- * Revision 1.15  2003/12/31 07:03:47  npetroni
- * made a number of changes to the config code to generalize handling of EAP
- * methods and phase2. I still need to go back and make the parser work for
- * other phase2 type in PEAP, but the backend is there.
- *
- * Revision 1.14  2003/12/28 20:41:57  chessing
- *
- * Added support for EAP-GTC.  It is the exact same code as OTP, so only the EAP type and defines in eap.c were needed.
- *
- * Revision 1.13  2003/12/28 07:13:21  chessing
- *
- * Fixed a problem where we would segfault on an EAP type we didn't understand.  Added EAP-OTP.  EAP-OTP has been tested using the opie package, and Radiator 3.8.  EAP-OTP currently prompts for a passphrase, which it shouldn't do, so it should be considered *VERY* much in test mode until we finish the GUI.
- *
- * Revision 1.12  2003/12/19 23:19:11  npetroni
- * updated config code and test example. Fixed a couple things
- *   1. added new variables to globals:
- *      startup_command
- *      first_auth_command
- *      reauth_command
- *      auth_period
- *      held_period
- *      max_starts
- *      allow_interfaces
- *      deny_ineterfaces
- *
- *   2. added new variables to network:
- *      dest_mac
- *
- *   3. added new variables to ttls:
- *      phase2_type
- *
- *   4. added new variables to peap:
- *      allow_types
- *
- *   5. layed the groundwork for "preferred types" to be sent in Nak
- *
- * Revision 1.11  2003/12/14 06:11:03  chessing
- *
- * Fixed some stuff with SIM in relation to the new config structures.  Cleaned out CR/LF from LEAP source files.  Added user certificate support to TTLS and PEAP. Some additions to the IPC code. (Not tested yet.)
- *
- * Revision 1.10  2003/12/10 14:13:16  npetroni
- * updated configuration code to parse all types. example updated as well
- *
- * Revision 1.9  2003/12/07 06:20:19  chessing
- *
- * Changes to deal with new config file style.  Beginning of IPC code.
- *
- * Revision 1.8  2003/12/04 04:36:24  chessing
- *
- * Added support for multiple interfaces (-D now works), also added DEBUG_EXCESSIVE to help clean up some of the debug output (-d 6).
- *
- * Revision 1.7  2003/11/29 04:46:02  chessing
- *
- * EAP-SIM changes : EAP-SIM will now try to use the IMSI as the username, when the preferred EAP type is SIM, and the username value is NULL.  Also, if simautogen is TRUE, then we will also build and attach a realm as specified in the RFC.
- *
- * Revision 1.6  2003/11/29 03:50:03  chessing
- *
- * Added NAK code, EAP Type checking, split out daemon config from user config, added Display of EAP-Notification text, revamped phase 2 selection method for TTLS.
- *
- * Revision 1.5  2003/11/29 01:11:30  npetroni
- * Added first round of configuration code.
- * Structural Changes:
- *    added examle config file and finished config-parser to test configuration
- *    files and optionally dump the output
- *
- * Current Status:
- *   Have not added parameters for any other method than TLS so we can discuss
- *   the changes before doing so.
- *
- *   Did not update config_build() so chris can keep testing as before.
- *
- * Revision 1.4  2003/11/27 02:33:25  chessing
- *
- * Added LEAP code from Marios Karagiannopoulos.  Keying still needs to be completed.
- *
- * Revision 1.3  2003/11/24 02:14:08  chessing
- *
- * Added EAP-SIM (draft 11 still needs work), various small changes to eap calls, new hex dump code including ASCII dump (used mostly for dumping frames)
- *
- * Revision 1.2  2003/11/19 04:23:18  chessing
- *
- * Updates to fix the import
+ * Revision 1.33  2004/06/15 03:22:17  chessing
  *
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -239,6 +111,11 @@
 #include "xsup_debug.h"
 #include "xsup_err.h"
 #include "config.h"
+#include "wpa.h"
+
+#ifdef EAP_SIM_ENABLE
+#include "winscard.h"
+#endif
 
 // there has GOT to be a better way than this...
 #include "eap_types/md5/eapmd5.h"
@@ -247,16 +124,12 @@
 #include "eap_types/mschapv2/eapmschapv2.h"
 #include "eap_types/peap/eappeap.h"
 #include "eap_types/leap/eapleap.h"
-
-#ifndef EAP_SIM_ENABLE
-#define EAP_SIM_ENABLE
-#endif 
 #include "eap_types/sim/eapsim.h"
-
+#include "eap_types/aka/eapaka.h"
 
 
 struct config_data *config_info = NULL; // the configuration data structure
-int config_parse_debug = 0;
+int config_parse_debug = 1;
 extern FILE *yyin;
 extern int config_linenum;
 extern int yyparse(void);
@@ -316,7 +189,6 @@ struct config_network *config_find_network(struct config_network *nethead,
 
   if ((nethead == NULL) || (matchname == NULL))
     {
-      debug_printf(DEBUG_EVERYTHING, "No match name to search!\n");
       return NULL;
     }
 
@@ -338,7 +210,6 @@ struct config_network *config_find_network(struct config_network *nethead,
     {
       while ((cur != NULL) && (strcmp(cur->ssid, matchname) != 0))
 	{
-	  debug_printf(DEBUG_NORMAL, "%s ? %s\n", cur->ssid, matchname);
 	  cur = cur->next;
 	}
   
@@ -348,9 +219,40 @@ struct config_network *config_find_network(struct config_network *nethead,
 	  return cur;
 	}
     }
+
   return NULL;
 }
 
+/****************************************
+ *
+ * Get the priority value based on a network name.
+ *
+ ****************************************/
+int config_get_priority(char *network_name)
+{
+  struct config_network *result;
+
+  if (config_info == NULL) 
+    debug_printf(DEBUG_NORMAL, "No valid configuration information to search!\n");
+
+  if (network_name == NULL)
+    debug_printf(DEBUG_NORMAL, "Invalid network name passed to %s.\n",
+		 __FUNCTION__);
+
+  result = config_find_network(config_info->networks, network_name);
+
+  if (result == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Couldn't find configuration information for network %s!\n", network_name);
+      return 0;
+    }
+
+  debug_printf(DEBUG_CONFIG, "Returning Priority of %d\n", 
+	       result->priority);
+
+  return result->priority;
+}
+
 /****************************************
  *
  * Get configuration information out of memory, and populate the userdata
@@ -390,6 +292,38 @@ struct config_network *config_build(char *network_name)
   return NULL;
 }
 
+/*********************************************
+ *
+ * Determine if the interface given is in the list of interfaces.
+ *
+ *********************************************/
+int config_int_in_list(char *intName, struct config_string_list *strList)
+{
+  struct config_string_list *cur;
+
+  if ((!intName) || (!strList))
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid data passed in to int_in_list()!\n");
+      return XEMALLOC;
+    }
+
+  // Our check is case sensative!
+  cur = strList;
+
+  while ((cur != NULL) && (strcmp(intName, cur->name) != 0))
+    {
+      cur = cur->next;
+    }
+
+  if (cur == NULL)
+    {
+      return FALSE;    // The interface isn't in our list.
+    } 
+
+  return TRUE;
+}
+
+
 /************************************
  *
  * Set statemachine/config related variables for this interface.
@@ -482,6 +416,26 @@ char *config_get_reauth_cmd()
   return config_info->globals->reauth_command;
 }
 
+char config_get_allmulti()
+{
+  if ((config_info == NULL) || (config_info->globals == NULL))
+    {
+      debug_printf(DEBUG_CONFIG, "No configuration information available!  ALLMULTI will return default value!\n");
+      return 1;
+    }
+  return config_info->globals->allmulti;
+}
+
+char config_get_association()
+{
+  if ((config_info == NULL) || (config_info->globals == NULL))
+    {
+      debug_printf(DEBUG_CONFIG, "No configuration information available!  ASSOCIATION will return default value!\n");
+      return 0;
+    }
+  return config_info->globals->association;
+}
+
 /**********************************************************************************
  * NOTE: Do *NOT* debug_printf() in this function or you'll cause a recursive loop.
  **********************************************************************************/
@@ -569,6 +523,43 @@ int config_allows_network(struct config_data *conf, char *netname)
 // not call these from outside config code
 //**********************************************
 
+  /*******************/
+ /* CONFIG_WPA_PSK  */
+/*******************/
+
+void delete_config_wpa_psk(struct config_wpa_psk **tmp_wpa_psk)
+{
+  if (*tmp_wpa_psk == NULL)
+    return;
+
+  FREE_STRING((*tmp_wpa_psk)->key);
+  FREE_STRING((*tmp_wpa_psk)->hex_key);
+
+  free(*tmp_wpa_psk);
+  *tmp_wpa_psk = NULL;
+}
+
+void initialize_config_wpa_psk(struct config_wpa_psk **tmp_wpa_psk)
+{
+  if (*tmp_wpa_psk != NULL) {
+    delete_config_wpa_psk(tmp_wpa_psk);
+  }
+  *tmp_wpa_psk = 
+    (struct config_wpa_psk *)malloc(sizeof(struct config_wpa_psk));
+  if (*tmp_wpa_psk)
+    memset(*tmp_wpa_psk, 0, sizeof(struct config_wpa_psk));
+}
+
+void dump_config_wpa_psk(struct config_wpa_psk *tmp_wpa_psk)
+{
+  if (!tmp_wpa_psk)
+    return;
+  debug_printf(DEBUG_NORMAL, "\t---------------wpa-psk--------------\n");
+  debug_printf(DEBUG_NORMAL, "\t  ASCII key: \"%s\"\n", tmp_wpa_psk->key);
+  debug_printf(DEBUG_NORMAL, "\t  HEX key: \"%s\"\n", tmp_wpa_psk->hex_key);
+  debug_printf(DEBUG_NORMAL, "\t------------------------------------\n");
+}
+
   /*******************/
  /* CONFIG_TLS      */
 /*******************/
@@ -1190,6 +1181,9 @@ void dump_config_eap_peap(struct config_eap_peap *peap)
       debug_printf_nl(DEBUG_NORMAL, "NO\n");
       break;
     }
+   debug_printf(DEBUG_NORMAL, "\t  PEAP IAS_QUIRK: \"%s\"\n", 
+		 peap->ias_quirk ? "Yes" : "No");
+    
   if (TEST_FLAG(peap->flags, CONFIG_PEAP_ALLOW_MSCV2))
     debug_printf(DEBUG_NORMAL,"\t   Allow Phase 2 Type: MSCHAPv2\n");
   if (TEST_FLAG(peap->flags, CONFIG_PEAP_ALLOW_MD5))
@@ -1254,6 +1248,55 @@ void dump_config_eap_sim(struct config_eap_sim *sim, int level)
   }
 }
 
+  /*******************/
+ /* CONFIG_AKA      */
+/*******************/
+void delete_config_eap_aka(struct config_eap_aka **tmp_aka)
+{
+  if (*tmp_aka == NULL)
+    return;
+
+  FREE_STRING((*tmp_aka)->username);
+  FREE_STRING((*tmp_aka)->password);
+
+  free (*tmp_aka);
+  *tmp_aka = NULL;
+}
+
+void initialize_config_eap_aka(struct config_eap_aka **tmp_aka)
+{
+  if (*tmp_aka != NULL) {
+    delete_config_eap_aka(tmp_aka);
+  }
+  *tmp_aka = 
+    (struct config_eap_aka *)malloc(sizeof(struct config_eap_aka));  
+  if (*tmp_aka)
+    memset(*tmp_aka, 0, sizeof(struct config_eap_aka));
+}
+
+void dump_config_eap_aka(struct config_eap_aka *aka, int level)
+{
+  if (!aka)
+    return;
+  if (level == 0) {
+    debug_printf(DEBUG_NORMAL, "\t---------------eap-aka--------------\n");
+    debug_printf(DEBUG_NORMAL, "\t  AKA User: \"%s\"\n", aka->username);
+    debug_printf(DEBUG_NORMAL, "\t  AKA Pass: \"%s\"\n", aka->password);
+    debug_printf(DEBUG_NORMAL, "\t  AKA Auto Realm: %s\n",  
+		 aka->auto_realm ? "yes" : "no");  
+    debug_printf(DEBUG_NORMAL, "\t------------------------------------\n");
+  } else {
+    debug_printf(DEBUG_NORMAL, "\t\t^ ^ ^  eap-aka  ^ ^ ^\n");
+    debug_printf(DEBUG_NORMAL, "\t\t  AKA User: \"%s\"\n", 
+		 aka->username);
+    debug_printf(DEBUG_NORMAL, "\t\t  AKA Pass: \"%s\"\n", 
+		 aka->password);
+    debug_printf(DEBUG_NORMAL, "\t\t  AKA Auto Realm: %s\n",  
+		 aka->auto_realm ? "yes" : "no");  
+    debug_printf(DEBUG_NORMAL, "\t\t^ ^ ^ ^ ^ ^ ^ ^ ^ ^ ^ ^ ^ ^\n");
+  }
+}
+
 
   /*********************/
  /* CONFIG_EAP_METHOD */
@@ -1295,6 +1338,9 @@ void delete_config_eap_method(struct config_eap_method **method)
   if (*method == NULL)
     return;
   switch ((*method)->method_num) {
+  case WPA_PSK:
+    delete_config_wpa_psk((struct config_wpa_psk **)&((*method)->method_data));
+    break;
   case EAP_TYPE_TLS:
     delete_config_eap_tls((struct config_eap_tls **)&((*method)->method_data));
     break;
@@ -1304,9 +1350,16 @@ void delete_config_eap_method(struct config_eap_method **method)
   case EAP_TYPE_PEAP:
     delete_config_eap_peap((struct config_eap_peap **)&(*method)->method_data);
     break;
+
+#ifdef EAP_SIM_ENABLE
   case EAP_TYPE_SIM:
     delete_config_eap_sim((struct config_eap_sim **)&(*method)->method_data);
     break;
+  case EAP_TYPE_AKA:
+    delete_config_eap_aka((struct config_eap_aka **)&(*method)->method_data);
+    break;
+#endif
+
   case EAP_TYPE_TTLS:
     delete_config_eap_ttls((struct config_eap_ttls **)&(*method)->method_data);
     break; 
@@ -1332,6 +1385,9 @@ void dump_config_eap_method(struct config_eap_method *method, int dumplevel)
   if (method == NULL)
     return;
   switch ((method)->method_num) {
+  case WPA_PSK:
+    dump_config_wpa_psk((struct config_wpa_psk *)((method)->method_data));
+    break;
   case EAP_TYPE_TLS:
     dump_config_eap_tls((struct config_eap_tls *)((method)->method_data));
     break;
@@ -1342,10 +1398,18 @@ void dump_config_eap_method(struct config_eap_method *method, int dumplevel)
   case EAP_TYPE_PEAP:
     dump_config_eap_peap((struct config_eap_peap *)(method)->method_data);
     break;
+
+#ifdef EAP_SIM_ENABLE
   case EAP_TYPE_SIM:
     dump_config_eap_sim((struct config_eap_sim *)(method)->method_data,
 			dumplevel);
     break;
+  case EAP_TYPE_AKA:
+    dump_config_eap_aka((struct config_eap_aka *)(method)->method_data,
+			dumplevel);
+    break;
+#endif
+
   case EAP_TYPE_TTLS:
     dump_config_eap_ttls((struct config_eap_ttls *)(method)->method_data);
     break; 
@@ -1459,6 +1523,32 @@ void dump_config_network(struct config_network *net)
   else
     debug_printf(DEBUG_NORMAL, "  Wireless Control: NO\n");
 
+  debug_printf(DEBUG_NORMAL, "  Crypt Method: ");
+  switch (net->crypt_type)
+    {
+    case CRYPT_ANY:
+      debug_printf(DEBUG_NORMAL, "ANY\n");
+      break;
+
+    case CRYPT_WEP:
+      debug_printf(DEBUG_NORMAL, "WEP\n");
+      break;
+
+    case CRYPT_WPA_TKIP:
+      debug_printf(DEBUG_NORMAL, "WPA-TKIP\n");
+      break;
+      
+    case CRYPT_WPA_CCMP:
+      debug_printf(DEBUG_NORMAL, "WPA-CCMP\n");
+      break;
+
+    case CRYPT_WPA_ANY:
+      debug_printf(DEBUG_NORMAL, "WPA-ANY\n");
+      break;
+    }
+
+  if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_WPA_PSK))
+    debug_printf(DEBUG_NORMAL, "  Allow Type: WPA-PSK\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_TLS))
     debug_printf(DEBUG_NORMAL, "  Allow Type: TLS\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_MD5))
@@ -1473,11 +1563,15 @@ void dump_config_network(struct config_network *net)
     debug_printf(DEBUG_NORMAL, "  Allow Type: PEAP\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_SIM))
     debug_printf(DEBUG_NORMAL, "  Allow Type: SIM\n");
+  if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_AKA))
+    debug_printf(DEBUG_NORMAL, "  Allow Type: AKA\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_GTC))
     debug_printf(DEBUG_NORMAL, "  Allow Type: GTC\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_OTP))
     debug_printf(DEBUG_NORMAL, "  Allow Type: OTP\n");
 
+  if (TEST_FLAG(net->flags, CONFIG_NET_PREFER_WPA_PSK))
+    debug_printf(DEBUG_NORMAL, "  Prefer Type: WPA-PSK\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_PREFER_TLS))
     debug_printf(DEBUG_NORMAL, "  Prefer Type: TLS\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_PREFER_MD5))
@@ -1492,6 +1586,8 @@ void dump_config_network(struct config_network *net)
     debug_printf(DEBUG_NORMAL, "  Prefer Type: PEAP\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_PREFER_SIM))
     debug_printf(DEBUG_NORMAL, "  Prefer Type: SIM\n");
+  if (TEST_FLAG(net->flags, CONFIG_NET_PREFER_AKA))
+    debug_printf(DEBUG_NORMAL, "  Prefer Type: AKA\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_PREFER_GTC))
     debug_printf(DEBUG_NORMAL, "  Prefer Type: GTC\n");
   if (TEST_FLAG(net->flags, CONFIG_NET_PREFER_OTP))
@@ -1499,6 +1595,7 @@ void dump_config_network(struct config_network *net)
 
   debug_printf(DEBUG_NORMAL, "  SSID: \"%s\"\n", net->ssid);
   debug_printf(DEBUG_NORMAL, "  Identity: \"%s\"\n", net->identity);
+  debug_printf(DEBUG_NORMAL, "  Priority: %d\n", net->priority);
 
   if (TEST_FLAG(net->flags, CONFIG_NET_DEST_MAC))
     debug_printf(DEBUG_NORMAL, "  DEST MAC: %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n",
@@ -1612,6 +1709,9 @@ void initialize_config_globals(struct config_globals **tmp_globals)
     (struct config_globals *)malloc(sizeof(struct config_globals));  
   if (*tmp_globals)
     memset(*tmp_globals, 0, sizeof(struct config_globals));
+
+  // We want to set allmulti to have a default of 1.
+  (*tmp_globals)->allmulti = 1;
 }
 
 void dump_config_globals(struct config_globals *globals)
@@ -1646,6 +1746,16 @@ void dump_config_globals(struct config_globals *globals)
   if (TEST_FLAG(globals->flags, CONFIG_GLOBALS_MAX_STARTS))
     debug_printf(DEBUG_NORMAL,"Max Starts: %d\n", globals->max_starts);
 
+  if (globals->allmulti == 1)
+    debug_printf(DEBUG_NORMAL, "ALLMULTI is enabled!\n");
+  else
+    debug_printf(DEBUG_NORMAL, "ALLMULTI is disabled!\n");
+
+  if (globals->association == 0)
+    debug_printf(DEBUG_NORMAL, "ASSOCIATION is set to AUTO!\n");
+  else
+    debug_printf(DEBUG_NORMAL, "ASSOCIATION is set to MANUAL!\n");
+
   if (globals->allow_interfaces)
     dump_config_string_list(globals->allow_interfaces, "Interface Allowed");
   else 
