@@ -59,86 +59,20 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eapmschapv2.c,v 1.20 2004/04/26 20:51:14 chessing Exp $
- * $Date: 2004/04/26 20:51:14 $
+ * $Id: eapmschapv2.c,v 1.23 2004/06/29 01:35:35 chessing Exp $
+ * $Date: 2004/06/29 01:35:35 $
  * $Log: eapmschapv2.c,v $
- * Revision 1.20  2004/04/26 20:51:14  chessing
+ * Revision 1.23  2004/06/29 01:35:35  chessing
  *
- * Patch to attempt to fix the init_interface_* errors reported on the list.  Removed password clearing on failed authentication attempts.  Password clearing currently has some issues that will prevent it from being in the 1.0 stable.
+ * Added credit to eapleap.c for Gilbert Goodwill (who gave me the clues I needed to figure out the keying piece for LEAP).  Added patch from Toby Collett to make PEAP work for broken implementations of IAS.  (New option ias_quirk has been added.)
  *
- * Revision 1.19  2004/04/13 22:13:29  chessing
+ * Revision 1.22  2004/06/15 03:35:20  chessing
  *
- * Additional error checking in all eap methods.
+ * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
  *
- * Revision 1.18  2004/04/06 20:31:26  chessing
+ * Revision 1.21  2004/06/15 03:22:29  chessing
  *
- * PEAP NOW WORKS WITH IAS!!!!!! (Thanks to help from Matthew Gast!! (We love you! ;))  Also, added patches from yesterday's testing at iLabs, including some keying fixes, some segfault fixes, and a few other misc. issues.  iLabs testing has been worth it!
- *
- * Revision 1.17  2004/04/05 17:19:17  chessing
- *
- * Added additional checks against pointers to try to help prevent segfaults.  (This still needs to be completed.)  Fixed a problem with PEAP where a NULL input packet would result in a huge unencrypted packet, and a segfault.  (This was triggered when using one of the gui password tools.  When the password was in the config file, it wouldn't be triggered.)
- *
- * Revision 1.16  2004/04/02 20:50:20  chessing
- *
- * Attempt to fix PEAP with IAS. At this point, we can get through the TLS piece of the PEAP authentication, and successfully attempt a phase 2 authentication.  But, for some reason MS-CHAPv2 is failing when used with IAS.  (But at least we are one step closer!)  Also, removed the des pieces that were needed for eap-mschapv2, since we can use the OpenSSL routines instead.  The proper way to handle DES was found while looking at the CVS code for wpa_supplicant.  The fix for phase 1 of PEAP was found while looking at the commit notes for wpa_supplicant.  (wpa_supplicant is part of hostap, and is written/maintained by Jouni Malinen.)
- *
- * Revision 1.15  2004/03/22 05:33:47  chessing
- * Fixed some potential issues with the example config in etc.  Fixed several memory leaks in various locations.  Re-tested all EAP types except SIM/OTP/GTC/LEAP.  (Those test will happen this next week.) Getting close to a 1.0pre release!
- *
- * Revision 1.14  2004/03/19 23:43:56  chessing
- *
- * Lots of changes.  Changed the password prompting code to no longer require the EAP methods to maintain their own stale frame buffer.  (Frame buffer pointers should be moved out of generic_eap_data before a final release.)  Instead, EAP methods should set need_password in generic_eap_data to 1, along with the variables that identify the eap type being used, and the challenge data (if any -- only interesting to OTP/GTC at this point).  Also fixed up xsup_set_pwd.c, and got it back in CVS.  (For some reason, it was in limbo.)  Added xsup_monitor under gui_tools/cli.  xsup_monitor will eventually be a cli program that will monitor XSupplicant (running as a daemon) and display status information, and request passwords when they are not in the config.
- *
- * Revision 1.13  2004/02/16 05:06:00  chessing
- *
- * Added support for the deny_interfaces, and allow_interfaces options in the config file.  (We should now have support for *EVERYTHING* in the new config file format!)  Updated EAP types other than SIM to use the new password prompt/delay code.  Phase 2 of TTLS still needs to be completed, along with the code that actually notifies the GUI.
- *
- * Revision 1.12  2004/02/06 06:13:31  chessing
- *
- * Cleaned up some unneeded stuff in the configure.in file as per e-mail from Rakesh Patel.  Added all 12 patches from Jouni Malinen (Including wpa_supplicant patch, until we can add true wpa support in xsupplicant.)
- *
- * Revision 1.11  2004/01/20 05:57:05  chessing
- *
- * All EAP types except PEAP and TTLS now support having their passwords sent in via the command line program.  (This means no more gets() call in OTP!)  A config_eap_otp structure was created in config.h to support GTC/OTP.  We need to define an eap_otp and eap_gtc config section.  Since both require some kind of information be presented there are no attributes that need to be defined in their part of the configuration.
- *
- * Revision 1.10  2004/01/17 21:16:16  chessing
- *
- * Various segfault fixes.  PEAP now works correctly again.  Some new error checking in the tls handlers.  Fixes for the way we determine if we have changed ESSIDs.  We now quit when we don't have a config, or when the config is bad. Added code to check and see if a frame is in the queue, and don't sleep if there is.  "Fixed" ID issue by inheriting the ID from the parent where needed.  However, assigning an ID inside of a handler will override the parent ID.  This could cause problems with some EAP types.  We should add a "username" field to PEAP to allow configuration of the inner EAP identity.
- *
- * Revision 1.9  2004/01/15 23:45:11  chessing
- *
- * Fixed a segfault when looking for wireless interfaces when all we had was a wired interface.  Fixed external command execution so that junk doesn't end up in the processed string anymore.  Changed the state machine to call txRspAuth even if there isn't a frame to process.  This will enable EAP methods to request information from a GUI interface (such as passwords, or supply challenge information that might be needed to generate passwords).  EAP methods now must decide what to do when they are handed NULL for the pointer to the in frame.  If they don't need any more data, they should quietly exit.
- *
- * Revision 1.8  2004/01/15 01:12:45  chessing
- *
- * Fixed a keying problem (keying material wasn't being generated correctly).  Added support for global counter variables from the config file. (Such as auth_period)  Added support for executing command defined in the config file based on different events.  (Things such as what to do on reauth.)  Added the ability to roam to a different SSID.  We now check to make sure our BSSID hasn't changed, and we follow it, if it has.  Fixed a sefault when the program was terminated in certain states.  Added attempt at better garbage collection on program termination. Various small code cleanups.
- *
- * Revision 1.7  2004/01/14 05:44:48  chessing
- *
- * Added pid file support. (Very basic for now, needs to be improved a little.)  Attempted to add setup of global variables. (Need to figure out why it is segfaulting.)  Added more groundwork for IPC.
- *
- * Revision 1.6  2004/01/13 01:55:55  chessing
- *
- * Major changes to EAP related code.  We no longer pass in an interface_data struct to EAP handlers.  Instead, we hand in a generic_eap_data struct which containsnon-interface specific information.  This will allow EAP types to be reused as phase 2 type easier.  However, this new code may create issues with EAP types that make use of the identity in the eap type.  Somehow, the identity value needs to propigate down to the EAP method.  It currently does not.  This should be any easy fix, but more testing will be needed.
- *
- * Revision 1.5  2004/01/06 23:35:07  chessing
- *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.4  2003/11/22 06:10:38  chessing
- *
- * Changes to the eap type process calls, to remove a pointless parameter.
- *
- * Revision 1.3  2003/11/21 05:09:47  chessing
- *
- * PEAP now works!
- *
- * Revision 1.2  2003/11/20 00:05:32  chessing
- *
- * EAP-MSCHAPv2 now supports generation of keys.  (New feature)
- *
- * Revision 1.1.1.1  2003/11/19 04:13:27  chessing
- * New source tree
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -262,8 +196,17 @@ int eapmschapv2_process(struct generic_eap_data *thisint, u_char *dataoffs,
       // This value should *ALWAYS* be 16!
       if (challenge->Value_Size != 0x10)
 	{
-	  debug_printf(DEBUG_NORMAL, "(EAP-MS-CHAPv2) Invalid Value-Size! (%d)\n", challenge->Value_Size);
-	  return XEMSCHAPV2LEN;
+	  if (thisint->ias_quirk == 1)
+	    {
+	      debug_printf(DEBUG_NORMAL, "(EAP-MS-CHAPv2) Invalid Value-Size! (%d), forced to 0x10 (ias_quirk=yes)\n", challenge->Value_Size);
+	      challenge->Value_Size = 0x10;
+	    }
+	  else 
+	    {
+	      debug_printf(DEBUG_NORMAL, "(EAP-MS-CHAPv2) Invalid Value-Size! (%d)\n", challenge->Value_Size);
+	      debug_printf(DEBUG_NORMAL, "(EAP-MS-CHAPv2) Should you enable ias_quirk?\n");
+	      return XEMSCHAPV2LEN;
+	    }
 	}
 
       if (myvars->AuthenticatorChallenge != NULL)
@@ -411,6 +354,7 @@ int eapmschapv2_get_keys(struct interface_data *thisint)
   if (thisint->keyingMaterial == NULL) return -1;
 
   memcpy(thisint->keyingMaterial, myconf->keyingMaterial, 64);
+  thisint->keyingLength = 32;
   
   return XENONE;
 }
