@@ -59,97 +59,20 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eappeap.c,v 1.23 2004/04/14 21:09:32 chessing Exp $
- * $Date: 2004/04/14 21:09:32 $
+ * $Id: eappeap.c,v 1.26 2004/07/15 04:15:37 chessing Exp $
+ * $Date: 2004/07/15 04:15:37 $
  * $Log: eappeap.c,v $
- * Revision 1.23  2004/04/14 21:09:32  chessing
+ * Revision 1.26  2004/07/15 04:15:37  chessing
  *
- * Finished up extra error checking code.  Added ability to have passwords removed from memory on an authentication failure, so that a new password can be entered.  However, this feature has been disabled at this point due to a few small issues.  It will probably show up in 1.1. ;)  (It just isn't stable enough right now.)
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Revision 1.22  2004/04/06 20:31:27  chessing
+ * Revision 1.25  2004/06/15 03:35:20  chessing
  *
- * PEAP NOW WORKS WITH IAS!!!!!! (Thanks to help from Matthew Gast!! (We love you! ;))  Also, added patches from yesterday's testing at iLabs, including some keying fixes, some segfault fixes, and a few other misc. issues.  iLabs testing has been worth it!
+ * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
  *
- * Revision 1.21  2004/03/28 20:37:10  chessing
+ * Revision 1.24  2004/06/15 03:22:31  chessing
  *
- * PEAP session resumption now works.
- *
- * Revision 1.20  2004/03/28 06:07:17  chessing
- * Added failure call to EAP methods to enable context resets for TLS based authentication protocols.  The resets are needed if an authentiction attempt fails, and we have session resumption enabled.  However, resetting it when we aren't using session resumption won't hurt anything, and probably isn't a bad idea.  The new failure handler can also be used to destroy passwords after a failed attempt, which will then cause xsupplicant to request another password from any listening GUIs. TLS session resumption is enabled (and works) for TLS and TTLS.  PEAP loops forever, and needs to be reviewed.
- *
- * Revision 1.19  2004/03/26 03:52:47  chessing
- *
- * Fixed a bug in xsup_debug that would cause config-parse to crash.  Added new key word for session resumption.  Added code to attempt session resumption.  So far, testing has not succeeded, but it is attempting resume. (Four TTLS packets are exchanged, and then we get a failure.)  More testing is needed.
- *
- * Revision 1.18  2004/03/22 05:33:47  chessing
- * Fixed some potential issues with the example config in etc.  Fixed several memory leaks in various locations.  Re-tested all EAP types except SIM/OTP/GTC/LEAP.  (Those test will happen this next week.) Getting close to a 1.0pre release!
- *
- * Revision 1.17  2004/03/19 23:43:57  chessing
- *
- * Lots of changes.  Changed the password prompting code to no longer require the EAP methods to maintain their own stale frame buffer.  (Frame buffer pointers should be moved out of generic_eap_data before a final release.)  Instead, EAP methods should set need_password in generic_eap_data to 1, along with the variables that identify the eap type being used, and the challenge data (if any -- only interesting to OTP/GTC at this point).  Also fixed up xsup_set_pwd.c, and got it back in CVS.  (For some reason, it was in limbo.)  Added xsup_monitor under gui_tools/cli.  xsup_monitor will eventually be a cli program that will monitor XSupplicant (running as a daemon) and display status information, and request passwords when they are not in the config.
- *
- * Revision 1.16  2004/03/17 21:21:40  chessing
- *
- * Hopefully xsup_set_pwd is in the right place now. ;)  Added the functions needed for xsupplicant to request a password from a GUI client.  (Still needs to be tested.)  Updated TTLS and PEAP to support password prompting.  Fixed up curState change in statemachine.c, so it doesn't print [ALL] in front of the current state.
- *
- * Revision 1.15  2004/03/15 16:23:24  chessing
- *
- * Added some checks to TLS using EAP types to make sure the root certificate isn't set to NULL.  (If it is, we can't authenticate, so we bail out.)  Changed the user certificate settings in the config file to all start with user_.  So, "cert" is now "user_cert", "key" is now "user_key", and "key_pass" is now "user_key_pass".  The structures and other related variables were also updated to reflect this change.  THIS WILL PROBABLY BREAK CONFIG FILES FOR SOME USERS!  (Be prepared for complaints on the list!)  ;)
- *
- * Revision 1.14  2004/03/05 23:58:45  chessing
- *
- * Added CN (sometimes called server name) checking to TTLS and PEAP.  This resulted in two new config options in the eap-ttls, and eap-peap blocks.  cncheck should be the name (or partial name) to match in the CN.  cnexact should be yes/no depending on if we want to match the CN exactly, or just see if our substring is in the CN.
- *
- * Revision 1.13  2004/03/02 01:03:53  chessing
- *
- * Added Jari Ahonen's SSL verification callback.  Added support to PEAP and TTLS to turn off certificate validation checking by setting the root_cert variable in the config to NONE.  (Case sensative!)  We will also display a warning when running in this mode.  Added initial hooks to support certificate CN checking.
- *
- * Revision 1.12  2004/02/16 05:06:00  chessing
- *
- * Added support for the deny_interfaces, and allow_interfaces options in the config file.  (We should now have support for *EVERYTHING* in the new config file format!)  Updated EAP types other than SIM to use the new password prompt/delay code.  Phase 2 of TTLS still needs to be completed, along with the code that actually notifies the GUI.
- *
- * Revision 1.11  2004/02/06 06:13:31  chessing
- *
- * Cleaned up some unneeded stuff in the configure.in file as per e-mail from Rakesh Patel.  Added all 12 patches from Jouni Malinen (Including wpa_supplicant patch, until we can add true wpa support in xsupplicant.)
- *
- * Revision 1.10  2004/01/15 23:45:11  chessing
- *
- * Fixed a segfault when looking for wireless interfaces when all we had was a wired interface.  Fixed external command execution so that junk doesn't end up in the processed string anymore.  Changed the state machine to call txRspAuth even if there isn't a frame to process.  This will enable EAP methods to request information from a GUI interface (such as passwords, or supply challenge information that might be needed to generate passwords).  EAP methods now must decide what to do when they are handed NULL for the pointer to the in frame.  If they don't need any more data, they should quietly exit.
- *
- * Revision 1.9  2004/01/15 01:12:45  chessing
- *
- * Fixed a keying problem (keying material wasn't being generated correctly).  Added support for global counter variables from the config file. (Such as auth_period)  Added support for executing command defined in the config file based on different events.  (Things such as what to do on reauth.)  Added the ability to roam to a different SSID.  We now check to make sure our BSSID hasn't changed, and we follow it, if it has.  Fixed a sefault when the program was terminated in certain states.  Added attempt at better garbage collection on program termination. Various small code cleanups.
- *
- * Revision 1.8  2004/01/14 05:44:48  chessing
- *
- * Added pid file support. (Very basic for now, needs to be improved a little.)  Attempted to add setup of global variables. (Need to figure out why it is segfaulting.)  Added more groundwork for IPC.
- *
- * Revision 1.7  2004/01/13 01:55:56  chessing
- *
- * Major changes to EAP related code.  We no longer pass in an interface_data struct to EAP handlers.  Instead, we hand in a generic_eap_data struct which containsnon-interface specific information.  This will allow EAP types to be reused as phase 2 type easier.  However, this new code may create issues with EAP types that make use of the identity in the eap type.  Somehow, the identity value needs to propigate down to the EAP method.  It currently does not.  This should be any easy fix, but more testing will be needed.
- *
- * Revision 1.6  2004/01/06 23:35:07  chessing
- *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.5  2003/12/14 06:11:03  chessing
- *
- * Fixed some stuff with SIM in relation to the new config structures.  Cleaned out CR/LF from LEAP source files.  Added user certificate support to TTLS and PEAP. Some additions to the IPC code. (Not tested yet.)
- *
- * Revision 1.4  2003/11/24 02:14:08  chessing
- *
- * Added EAP-SIM (draft 11 still needs work), various small changes to eap calls, new hex dump code including ASCII dump (used mostly for dumping frames)
- *
- * Revision 1.3  2003/11/22 06:10:38  chessing
- *
- * Changes to the eap type process calls, to remove a pointless parameter.
- *
- * Revision 1.2  2003/11/21 05:09:47  chessing
- *
- * PEAP now works!
- *
- * Revision 1.1.1.1  2003/11/19 04:13:26  chessing
- * New source tree
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -362,9 +285,10 @@ int eappeap_process(struct generic_eap_data *thisint, u_char *dataoffs,
 
   if (dataoffs == NULL) return XENONE;
 
-      /* PEAP adds some version bits to flags byte.  They need to be stripped out. */
+/* PEAP adds some version bits to flags byte.  They need to be stripped out. */
 
   peap_version = ((uint8_t)dataoffs[0] & 0x03);  // Get the version #.
+
   set_peap_version(p2d, peap_version);  // Tell PEAP what version we want to use.
   dataoffs[0] = ((uint8_t)dataoffs[0] & 0xfc);  // Mask out the version bits.
   
@@ -439,6 +363,7 @@ int eappeap_get_keys(struct interface_data *thisint)
       free(thisint->keyingMaterial);
     }
   thisint->keyingMaterial = tls_funcs_gen_keyblock(thisint->userdata->activemethod);
+  thisint->keyingLength = 32;
 
   if (thisint->keyingMaterial == NULL) return -1;
 
