@@ -59,66 +59,20 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: peap_phase2.c,v 1.16 2004/04/14 21:09:32 chessing Exp $
- * $Date: 2004/04/14 21:09:32 $
+ * $Id: peap_phase2.c,v 1.19 2004/07/15 04:15:37 chessing Exp $
+ * $Date: 2004/07/15 04:15:37 $
  * $Log: peap_phase2.c,v $
- * Revision 1.16  2004/04/14 21:09:32  chessing
+ * Revision 1.19  2004/07/15 04:15:37  chessing
  *
- * Finished up extra error checking code.  Added ability to have passwords removed from memory on an authentication failure, so that a new password can be entered.  However, this feature has been disabled at this point due to a few small issues.  It will probably show up in 1.1. ;)  (It just isn't stable enough right now.)
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Revision 1.15  2004/04/07 22:23:12  chessing
+ * Revision 1.18  2004/06/29 01:35:36  chessing
  *
- * Fixed a segfault when a phase 2 method wasn't defined for TTLS.  Also, fixed an issue with TTLS authentication with Funk's Steel-Belted Radius.  The Funk Server would claim that we send a connection termination message.  However, the issue was that we were sending a length value in our encrypted TLS packets, and it didn't like this.  (I am not sure if Funk uses the Microsoft Crypto Provider, but it may be a strange behavior in MCP.)
+ * Added credit to eapleap.c for Gilbert Goodwill (who gave me the clues I needed to figure out the keying piece for LEAP).  Added patch from Toby Collett to make PEAP work for broken implementations of IAS.  (New option ias_quirk has been added.)
  *
- * Revision 1.14  2004/04/06 20:31:27  chessing
+ * Revision 1.17  2004/06/15 03:22:31  chessing
  *
- * PEAP NOW WORKS WITH IAS!!!!!! (Thanks to help from Matthew Gast!! (We love you! ;))  Also, added patches from yesterday's testing at iLabs, including some keying fixes, some segfault fixes, and a few other misc. issues.  iLabs testing has been worth it!
- *
- * Revision 1.13  2004/04/05 17:19:30  chessing
- *
- * Added additional checks against pointers to try to help prevent segfaults.  (This still needs to be completed.)  Fixed a problem with PEAP where a NULL input packet would result in a huge unencrypted packet, and a segfault.  (This was triggered when using one of the gui password tools.  When the password was in the config file, it wouldn't be triggered.)
- *
- * Revision 1.12  2004/04/02 20:50:21  chessing
- *
- * Attempt to fix PEAP with IAS. At this point, we can get through the TLS piece of the PEAP authentication, and successfully attempt a phase 2 authentication.  But, for some reason MS-CHAPv2 is failing when used with IAS.  (But at least we are one step closer!)  Also, removed the des pieces that were needed for eap-mschapv2, since we can use the OpenSSL routines instead.  The proper way to handle DES was found while looking at the CVS code for wpa_supplicant.  The fix for phase 1 of PEAP was found while looking at the commit notes for wpa_supplicant.  (wpa_supplicant is part of hostap, and is written/maintained by Jouni Malinen.)
- *
- * Revision 1.11  2004/03/28 06:07:17  chessing
- * Added failure call to EAP methods to enable context resets for TLS based authentication protocols.  The resets are needed if an authentiction attempt fails, and we have session resumption enabled.  However, resetting it when we aren't using session resumption won't hurt anything, and probably isn't a bad idea.  The new failure handler can also be used to destroy passwords after a failed attempt, which will then cause xsupplicant to request another password from any listening GUIs. TLS session resumption is enabled (and works) for TLS and TTLS.  PEAP loops forever, and needs to be reviewed.
- *
- * Revision 1.10  2004/03/27 02:20:07  chessing
- *
- * Fixed a problem where the IPC socket wasn't getting deallocated correctly, and would keep xsupplicant from running a second time.  Added the needed hooks to make PEAP-GTC work.  (Not tested yet.)
- *
- * Revision 1.9  2004/03/22 05:33:47  chessing
- * Fixed some potential issues with the example config in etc.  Fixed several memory leaks in various locations.  Re-tested all EAP types except SIM/OTP/GTC/LEAP.  (Those test will happen this next week.) Getting close to a 1.0pre release!
- *
- * Revision 1.8  2004/03/17 21:21:40  chessing
- *
- * Hopefully xsup_set_pwd is in the right place now. ;)  Added the functions needed for xsupplicant to request a password from a GUI client.  (Still needs to be tested.)  Updated TTLS and PEAP to support password prompting.  Fixed up curState change in statemachine.c, so it doesn't print [ALL] in front of the current state.
- *
- * Revision 1.7  2004/02/06 06:13:31  chessing
- *
- * Cleaned up some unneeded stuff in the configure.in file as per e-mail from Rakesh Patel.  Added all 12 patches from Jouni Malinen (Including wpa_supplicant patch, until we can add true wpa support in xsupplicant.)
- *
- * Revision 1.6  2004/01/17 21:16:16  chessing
- *
- * Various segfault fixes.  PEAP now works correctly again.  Some new error checking in the tls handlers.  Fixes for the way we determine if we have changed ESSIDs.  We now quit when we don't have a config, or when the config is bad. Added code to check and see if a frame is in the queue, and don't sleep if there is.  "Fixed" ID issue by inheriting the ID from the parent where needed.  However, assigning an ID inside of a handler will override the parent ID.  This could cause problems with some EAP types.  We should add a "username" field to PEAP to allow configuration of the inner EAP identity.
- *
- * Revision 1.5  2004/01/13 01:55:56  chessing
- *
- * Major changes to EAP related code.  We no longer pass in an interface_data struct to EAP handlers.  Instead, we hand in a generic_eap_data struct which containsnon-interface specific information.  This will allow EAP types to be reused as phase 2 type easier.  However, this new code may create issues with EAP types that make use of the identity in the eap type.  Somehow, the identity value needs to propigate down to the EAP method.  It currently does not.  This should be any easy fix, but more testing will be needed.
- *
- * Revision 1.4  2004/01/06 23:35:07  chessing
- *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.3  2003/11/22 06:10:39  chessing
- *
- * Changes to the eap type process calls, to remove a pointless parameter.
- *
- * Revision 1.2  2003/11/21 05:09:47  chessing
- *
- * PEAP now works!
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -219,14 +173,12 @@ void peap_pad_frame(u_char *in, int in_size, u_char *out, int *out_size)
     }
 }
 
-void do_peap_version1(struct generic_eap_data *thisint, u_char *in, int in_size, 
+void do_peap_version1(struct phase2_data *thisint, u_char *in, int in_size, 
 		      u_char *out, int *out_size)
 {
-  char *new_frame = NULL, *username = NULL;
-  int eapvalue, new_frame_size;
-  uint16_t i;
+  char *new_frame = NULL;
+  int eapvalue, eapid;
   struct tls_vars *mytls_vars;
-  struct config_eap_peap *userdata;
 
   if ((!thisint) || (!out) || (!out_size))
     {
@@ -243,20 +195,12 @@ void do_peap_version1(struct generic_eap_data *thisint, u_char *in, int in_size,
   
   *out_size = 0;
 
-  /// XXXXXXXXX  Reconsider how this is done! (Phase 2 for PEAP in general.)
-  userdata = (struct config_eap_peap *)thisint->eap_conf_data;
-
-  if (!userdata)
-    {
-      debug_printf(DEBUG_NORMAL, "Invalid user configuration in do_peap_version1()!\n");
-      return;
-    }
-
-  mytls_vars = (struct tls_vars *)thisint->eap_data;
+  mytls_vars = (struct tls_vars *)thisint->eapdata->eap_data;
   // mytls_vars may be NULL here!  This is okay, as long as we aren't in the
   // middle of an inner authentication.
 
   eapvalue = in[4];
+  eapid = in[5]; // Inner EAP id.
 
   debug_printf(DEBUG_AUTHTYPES, "Inner packet : \n");
   if (in_size < 1522)
@@ -277,113 +221,21 @@ void do_peap_version1(struct generic_eap_data *thisint, u_char *in, int in_size,
   switch ((uint8_t)eapvalue)
     {
     case EAP_REQUEST:
-	// In version 1, we answer with an EAP header.
-      debug_printf(DEBUG_AUTHTYPES, "Got a Phase 2 EAP_REQUEST!\n");
-      out[0] = EAP_RESPONSE;
-      out[1] = in[1];  // Use the same ID #
-
-      username = thisint->identity;
-      i = htons(strlen(username)+5);
-      memcpy((char *)&out[2], (uint16_t *)&i, 2);  // The length of the username + header.
-      out[4] = EAP_TYPE_IDENTITY;
-      memcpy(&out[5], username, strlen(username)+1);
-      thisint->need_password = 0;
-      *out_size = strlen(username)+5;
-
+      eap_request_id(thisint->eapdata->identity, eapid, out, out_size); 
       break;
 
     case EAP_SUCCESS:
       printf("Got a phase 2 success!\n");
       break;
 
+      // If we catch the EAP_FAILURE, it has the same value as MD5.  Which
+      // is probably why PEAP-EAP-MD5 isn't allowed in XP. ;)
+      /*
     case EAP_FAILURE:
       printf("Got a phase 2 failure!\n");
       break;
-
-    default:
-    case EAP_TYPE_PEAP:  // Is this a PEAP inner request?
-      debug_printf(DEBUG_AUTHTYPES, "Got a phase 2 request for PEAP, NAKing!\n");
-      out[0] = EAP_RESPONSE;
-      out[1] = in[1];
-      out[2] = 0;
-      out[3] = 6;
-      out[4]=EAP_TYPE_NAK;  // NAK
-      out[5]=EAP_TYPE_MSCHAPV2; // MS-CHAPv2
-      *out_size = 6;
-      break;
-      /*
-    case EAP_TYPE_GTC:
-      debug_printf(DEBUG_AUTHTYPES, "Got a phase 2 request for GTC!\n");
-      if (thisint->eap_data == NULL)
-	{
-	  eapotp_setup(thisint);
-	  thisint->eapid = EAP_TYPE_GTC;
-	  debug_printf(DEBUG_AUTHTYPES, "(PEAP - Phase 2) Initialized GTC!\n");
-	}
-
-      if (!thisint->eap_data)
-	{
-	  debug_printf(DEBUG_NORMAL, "Invalid EAP state data in GTC section of do_peap_version1()!\n");
-	  return;
-	}
-
-      eapotp_process(thisint, (u_char *)&in[5], (in_size-5), new_frame, &new_frame_size);
-
-      if (thisint->need_password == 0)
-	{
-	  out[0] = EAP_RESPONSE;
-	  out[1] = in[1];
-
-	  i = ntohs(6+new_frame_size); // 6 bytes header, plus out answer
-	  memcpy(&out[2], (uint16_t *)&i, 2);
-	  out[4] = EAP_TYPE_GTC;  // We have a GTC answer
-
-	  memcpy(&out[5], new_frame, new_frame_size);
-	  *out_size = new_frame_size+5;
-	} else {
-	  *out_size = 0;
-	}
-      break;
       */
 
-    case EAP_TYPE_MSCHAPV2: 
-      debug_printf(DEBUG_AUTHTYPES, "Got a phase 2 request for MS-CHAPv2!\n");
-      if (thisint->eap_data == NULL)
-	{
-	  eapmschapv2_setup(thisint);
-	  thisint->eapid = EAP_TYPE_MSCHAPV2;
-	  debug_printf(DEBUG_AUTHTYPES, "(PEAP - Phase 2) Initalized MS-CHAPv2..\n");
-	}
-
-      /*      if (thisint->tempPwd != NULL)
-	{
-	  debug_printf(DEBUG_AUTHTYPES, "Temp Password : %s\n", thisint->tempPwd);
-	  }*/
-      if (!thisint->eap_data)
-	{
-	  debug_printf(DEBUG_NORMAL, "Invalid EAP state data in MS-CHAPv2 part of do_peap_version1()!\n");
-	  return;
-	}
-
-      eapmschapv2_process(thisint, (u_char *)&in[5], (in_size-5), new_frame, &new_frame_size);
-      
-      if ((thisint->need_password == 0) && ((new_frame_size > 0) && 
-					    (new_frame_size < 1522)))
-	{
-	  out[0] = EAP_RESPONSE;
-	  out[1] = in[1];
-
-	  i = ntohs(6+new_frame_size); // 6 bytes header, plus out answer
-	  memcpy(&out[2], (uint16_t *)&i, 2);
-	  out[4] = EAP_TYPE_MSCHAPV2;  // We have an MSCHAPv2 answer
-
-	  memcpy(&out[5], new_frame, new_frame_size);
-	  *out_size = new_frame_size+5;
-	} else {
-	  *out_size = 0;
-	}
-      break;
-
     case PEAP_EAP_EXTENSION: // EAP Extension
       debug_printf(DEBUG_AUTHTYPES, "Got an EAP extension frame!\n");
       out[0] = EAP_RESPONSE;
@@ -391,24 +243,31 @@ void do_peap_version1(struct generic_eap_data *thisint, u_char *in, int in_size,
       *out_size = in_size;
       break;
 
-      /*    default:
-      debug_printf(DEBUG_NORMAL, "Not sure how to handle this request! (%02X)\n", eapvalue);
-      *out_size = 0;
+    default:
+      if (eap_create_active_method(&thisint->activemethod,
+				   thisint->eapdata->identity,
+				   thisint->eapdata->tempPwd) != 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't build active method!  Phase 2 authentication will not happen!\n");
+	  return;
+	}
+      eap_request_auth(thisint->activemethod, thisint->eapdata->eap_conf_data,
+		       in, in_size, out, out_size);
       break;
-      */
     }
   free(new_frame);
 }
 
-void do_peap_version0(struct generic_eap_data *thisint, u_char *in, int in_size, 
+void do_peap_version0(struct phase2_data *thisint, u_char *in, int in_size, 
 		      u_char *out, int *out_size)
 {
   char *padded_frame, *new_frame;
   int padded_size, new_frame_size, eframe = 0;
 
-  if (!out_size)
+  if ((!thisint) || (!out_size))
     {
-      debug_printf(DEBUG_NORMAL, "Invalid pointer for out size!\n");
+      debug_printf(DEBUG_NORMAL, "Invalid data passed in to do_peap_version0()!\n");
+      if (out_size) *out_size = 0;
       return;
     }
 
@@ -442,7 +301,7 @@ void do_peap_version0(struct generic_eap_data *thisint, u_char *in, int in_size,
       return;
     }
 
-  if ((in[4] == 0x21) && (in[5] = 0x80))  
+  if ((in[4] == 0x21) && (in[5] = 0x80) && !(thisint->eapdata->ias_quirk == 1 && in[0] == EAP_TYPE_MSCHAPV2))  
     {
       eframe = 1;
       memcpy(padded_frame, in, in_size);
@@ -525,8 +384,9 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
       p2d->eapdata->eap_data = NULL;
     }
 
-  p2d->eapdata->eap_conf_data = peapconf->phase2->method_data;
+  p2d->eapdata->eap_conf_data = peapconf->phase2;
   p2d->eapdata->identity = thisint->identity;
+  p2d->eapdata->ias_quirk = peapconf->ias_quirk;
 
   decr_data = (char *)malloc(1550);
   if (decr_data == NULL) 
@@ -546,6 +406,9 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
   if (in_size > 0)
     {
       tls_crypt_decrypt(thisint, in, in_size, decr_data, &decrsize);
+
+      debug_printf(DEBUG_AUTHTYPES, "Decrypted dump : \n");
+      debug_hex_dump(DEBUG_AUTHTYPES, decr_data, decrsize);
     } else {
       free(decr_data);
       decr_data = NULL;
@@ -579,11 +442,11 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
     {
     case 0:
       debug_printf(DEBUG_AUTHTYPES, "Doing PEAP v0!\n");
-      do_peap_version0(p2d->eapdata, decr_data, decrsize, encr_data, &encrsize);
+      do_peap_version0(p2d, decr_data, decrsize, encr_data, &encrsize);
       break;
     case 1:
       debug_printf(DEBUG_AUTHTYPES, "Doing PEAP v1!\n");
-      do_peap_version1(p2d->eapdata, decr_data, decrsize, encr_data, &encrsize);
+      do_peap_version1(p2d, decr_data, decrsize, encr_data, &encrsize);
       break;
     default:
       debug_printf(DEBUG_NORMAL, "Unknown PEAP version!  (%d)\n",p2d->peap_version);
@@ -641,6 +504,5 @@ void peap_phase2_failed(struct generic_eap_data *thisint)
       return;
     }
 
-  // For now, we only support MS-CHAPv2, so we can call this.
-  eapmschapv2_failed(p2d->eapdata);
+  eap_do_fail(p2d->activemethod);
 }
