@@ -59,112 +59,20 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: tls_funcs.c,v 1.27 2004/05/04 00:42:48 chessing Exp $
- * $Date: 2004/05/04 00:42:48 $
+ * $Id: tls_funcs.c,v 1.30 2004/07/15 04:15:37 chessing Exp $
+ * $Date: 2004/07/15 04:15:37 $
  * $Log: tls_funcs.c,v $
- * Revision 1.27  2004/05/04 00:42:48  chessing
+ * Revision 1.30  2004/07/15 04:15:37  chessing
  *
- * Fix a bug in tls_funcs_load_random.
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Revision 1.26  2004/04/14 21:09:33  chessing
+ * Revision 1.29  2004/06/17 22:56:59  chessing
  *
- * Finished up extra error checking code.  Added ability to have passwords removed from memory on an authentication failure, so that a new password can be entered.  However, this feature has been disabled at this point due to a few small issues.  It will probably show up in 1.1. ;)  (It just isn't stable enough right now.)
+ * Fixed the problem where an attempt to execute an invalid program would result in a second instance of XSupplicant running.  (Which would then cause all kinds of interesting problems. ;)  Also added a bunch of additional debugging to the TLS code to try to resolve the strange stalls that sometimes happen in authentications that use TLS for phase 1.
  *
- * Revision 1.25  2004/04/13 22:13:31  chessing
+ * Revision 1.28  2004/06/15 03:22:32  chessing
  *
- * Additional error checking in all eap methods.
- *
- * Revision 1.24  2004/04/12 18:43:43  chessing
- *
- * A few small cosmetic fixups.
- *
- * Revision 1.23  2004/04/06 20:31:27  chessing
- *
- * PEAP NOW WORKS WITH IAS!!!!!! (Thanks to help from Matthew Gast!! (We love you! ;))  Also, added patches from yesterday's testing at iLabs, including some keying fixes, some segfault fixes, and a few other misc. issues.  iLabs testing has been worth it!
- *
- * Revision 1.22  2004/04/05 17:19:30  chessing
- *
- * Added additional checks against pointers to try to help prevent segfaults.  (This still needs to be completed.)  Fixed a problem with PEAP where a NULL input packet would result in a huge unencrypted packet, and a segfault.  (This was triggered when using one of the gui password tools.  When the password was in the config file, it wouldn't be triggered.)
- *
- * Revision 1.21  2004/03/28 20:37:10  chessing
- *
- * PEAP session resumption now works.
- *
- * Revision 1.20  2004/03/28 06:07:17  chessing
- * Added failure call to EAP methods to enable context resets for TLS based authentication protocols.  The resets are needed if an authentiction attempt fails, and we have session resumption enabled.  However, resetting it when we aren't using session resumption won't hurt anything, and probably isn't a bad idea.  The new failure handler can also be used to destroy passwords after a failed attempt, which will then cause xsupplicant to request another password from any listening GUIs. TLS session resumption is enabled (and works) for TLS and TTLS.  PEAP loops forever, and needs to be reviewed.
- *
- * Revision 1.19  2004/03/27 01:40:46  chessing
- *
- * Lots of small updates to free memory that wasn't getting freed, add some additional debug output, and fix a couple of memory leaks.
- *
- * Revision 1.18  2004/03/26 21:34:52  chessing
- * Fixed problem with interface being down on startup causing xsupplicant to not read the proper configuration information when the interface is brought up.  Added/fixed code to rebuild userdata piece of structure when the essid changes.  Added code to avoid setting a key on an interface if the interface doesn't already have encryption enabled.  Added a little bit of debugging code to help find a solution to an IPC socket problem.
- *
- * Revision 1.17  2004/03/26 03:52:52  chessing
- *
- * Fixed a bug in xsup_debug that would cause config-parse to crash.  Added new key word for session resumption.  Added code to attempt session resumption.  So far, testing has not succeeded, but it is attempting resume. (Four TTLS packets are exchanged, and then we get a failure.)  More testing is needed.
- *
- * Revision 1.16  2004/03/22 05:33:47  chessing
- * Fixed some potential issues with the example config in etc.  Fixed several memory leaks in various locations.  Re-tested all EAP types except SIM/OTP/GTC/LEAP.  (Those test will happen this next week.) Getting close to a 1.0pre release!
- *
- * Revision 1.15  2004/03/17 21:21:41  chessing
- *
- * Hopefully xsup_set_pwd is in the right place now. ;)  Added the functions needed for xsupplicant to request a password from a GUI client.  (Still needs to be tested.)  Updated TTLS and PEAP to support password prompting.  Fixed up curState change in statemachine.c, so it doesn't print [ALL] in front of the current state.
- *
- * Revision 1.14  2004/03/05 23:58:45  chessing
- *
- * Added CN (sometimes called server name) checking to TTLS and PEAP.  This resulted in two new config options in the eap-ttls, and eap-peap blocks.  cncheck should be the name (or partial name) to match in the CN.  cnexact should be yes/no depending on if we want to match the CN exactly, or just see if our substring is in the CN.
- *
- * Revision 1.13  2004/03/02 01:03:53  chessing
- *
- * Added Jari Ahonen's SSL verification callback.  Added support to PEAP and TTLS to turn off certificate validation checking by setting the root_cert variable in the config to NONE.  (Case sensative!)  We will also display a warning when running in this mode.  Added initial hooks to support certificate CN checking.
- *
- * Revision 1.12  2004/02/28 01:26:38  chessing
- *
- * Several critical updates.  Fixed the HMAC failure on some keys. (This was due to a lot more than just an off-by-one.)  Fixed up the key decryption routine to identify key packets with no encrypted key, and use the peer key instead.  When using the peer key, we also can handle packets that are padded funny.  (Our Cisco AP1200 has two null pad bytes at the end of some key frames.)  Changed the response ID function to not add a 00 to the end of the ID.  The 00 byte shouldn't have been seen by the RADIUS server unless they were not paying attention to the EAP-Length.  So, this wasn't really a bug fix.  Started to add support for CN checking for TLS based protocols.
- *
- * Revision 1.11  2004/02/06 06:13:32  chessing
- *
- * Cleaned up some unneeded stuff in the configure.in file as per e-mail from Rakesh Patel.  Added all 12 patches from Jouni Malinen (Including wpa_supplicant patch, until we can add true wpa support in xsupplicant.)
- *
- * Revision 1.10  2004/01/17 21:16:16  chessing
- *
- * Various segfault fixes.  PEAP now works correctly again.  Some new error checking in the tls handlers.  Fixes for the way we determine if we have changed ESSIDs.  We now quit when we don't have a config, or when the config is bad. Added code to check and see if a frame is in the queue, and don't sleep if there is.  "Fixed" ID issue by inheriting the ID from the parent where needed.  However, assigning an ID inside of a handler will override the parent ID.  This could cause problems with some EAP types.  We should add a "username" field to PEAP to allow configuration of the inner EAP identity.
- *
- * Revision 1.9  2004/01/15 01:12:45  chessing
- *
- * Fixed a keying problem (keying material wasn't being generated correctly).  Added support for global counter variables from the config file. (Such as auth_period)  Added support for executing command defined in the config file based on different events.  (Things such as what to do on reauth.)  Added the ability to roam to a different SSID.  We now check to make sure our BSSID hasn't changed, and we follow it, if it has.  Fixed a sefault when the program was terminated in certain states.  Added attempt at better garbage collection on program termination. Various small code cleanups.
- *
- * Revision 1.8  2004/01/14 22:07:25  chessing
- *
- * Fixes that were needed in order to allow us to authenticate correctly.  We should now be able to authenticate using only information provided by the config file!
- *
- * Revision 1.7  2004/01/13 01:55:56  chessing
- *
- * Major changes to EAP related code.  We no longer pass in an interface_data struct to EAP handlers.  Instead, we hand in a generic_eap_data struct which containsnon-interface specific information.  This will allow EAP types to be reused as phase 2 type easier.  However, this new code may create issues with EAP types that make use of the identity in the eap type.  Somehow, the identity value needs to propigate down to the EAP method.  It currently does not.  This should be any easy fix, but more testing will be needed.
- *
- * Revision 1.6  2004/01/06 23:35:08  chessing
- *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.5  2003/12/14 06:11:03  chessing
- *
- * Fixed some stuff with SIM in relation to the new config structures.  Cleaned out CR/LF from LEAP source files.  Added user certificate support to TTLS and PEAP. Some additions to the IPC code. (Not tested yet.)
- *
- * Revision 1.4  2003/12/07 06:20:20  chessing
- *
- * Changes to deal with new config file style.  Beginning of IPC code.
- *
- * Revision 1.3  2003/11/29 03:50:03  chessing
- *
- * Added NAK code, EAP Type checking, split out daemon config from user config, added Display of EAP-Notification text, revamped phase 2 selection method for TTLS.
- *
- * Revision 1.2  2003/11/21 05:09:47  chessing
- *
- * PEAP now works!
- *
- * Revision 1.1.1.1  2003/11/19 04:13:26  chessing
- * New source tree
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -176,6 +84,7 @@
 #include <netinet/in.h>
 #include <string.h>
 #include <inttypes.h>
+#include <unistd.h>
 #include "config.h"
 #include "profile.h"
 #include "eap.h"
@@ -218,6 +127,18 @@ char *get_cert_common_name(SSL *ssl_ctx)
   return commonName;
 }
 
+void tls_funcs_process_error()
+{
+  unsigned long err;
+
+  err = ERR_get_error();
+  if (err != 0)
+    {
+      debug_printf(DEBUG_NORMAL, "OpenSSL Error -- %s\n", 
+		   ERR_error_string(err, NULL));
+    }
+}
+
 static int ssl_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 {
   char buf[256];
@@ -264,6 +185,7 @@ int tls_funcs_init(struct generic_eap_data *thisint)
   if (mytls_vars->ctx == NULL)
     {
       debug_printf(DEBUG_NORMAL, "Couldn't initialize OpenSSL TLS library!\n");
+      tls_funcs_process_error();
       return XETLSINIT;
     }
 
@@ -274,8 +196,13 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
 {
   SSL_SESSION *sess = NULL;
   unsigned long err;
+  int counter;
 
-  if (mytls_vars == NULL) return XETLSSTARTFAIL;
+  if (mytls_vars == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "In tls_funcs_start(), mytls_vars is not valid!\n");
+      return XETLSSTARTFAIL;
+    }
 
   mytls_vars->resuming = 0;
 
@@ -285,13 +212,7 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
       if (!mytls_vars->ssl)
 	{
 	  debug_printf(DEBUG_NORMAL, "Couldn't create SSL object!\n");
-	  // First, make sure we don't have any errors.
-	  err = ERR_get_error();
-	  if (err != 0)
-	    {
-	      debug_printf(DEBUG_NORMAL, "OpenSSL Error -- %s\n", 
-			   ERR_error_string(err, NULL));
-	    }
+	  tls_funcs_process_error();
 
 	  return XETLSSTARTFAIL;
 	}
@@ -303,8 +224,10 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
 	  sess = SSL_get_session(mytls_vars->ssl);
 	  if (!sess)
 	    {
+	      debug_printf(DEBUG_AUTHTYPES, "Couldn't get session information!  We won't try to resume this session!\n");
 	      mytls_vars->resuming = 0;
 	    } else {
+	      debug_printf(DEBUG_AUTHTYPES, "Got session information, trying to resume session!\n");
 	      mytls_vars->resuming = 1;
 	    }
 	}
@@ -316,13 +239,30 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
       SSL_set_quiet_shutdown(mytls_vars->ssl, 1);
 
       // Now, close off our old session.
-      SSL_shutdown(mytls_vars->ssl);
+      err = 0;
+      counter = 0;
+      while ((err == 0) && (counter < 60))
+	{
+	  err = SSL_shutdown(mytls_vars->ssl);
+	  if (err == 0)
+	    {
+	      sleep(1);
+	      counter++;
+	    }
+	}
+
+      if (err < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Error trying to shut down SSL context data.\n");
+	  tls_funcs_process_error();
+	}
     }
 
   mytls_vars->ssl_in = BIO_new(BIO_s_mem());
   if (!mytls_vars->ssl_in)
     {
       debug_printf(DEBUG_NORMAL, "Couldn't create ssl_in!\n");
+      tls_funcs_process_error();
       return XETLSSTARTFAIL;
     }
 
@@ -330,6 +270,7 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
   if (!mytls_vars->ssl_out)
     {
       debug_printf(DEBUG_NORMAL, "Couldn't create ssl_out!\n");
+      tls_funcs_process_error();
       return XETLSSTARTFAIL;
     }
 
@@ -340,7 +281,11 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
       // If we have session information, we need to use it to resume the 
       // session.
       debug_printf(DEBUG_AUTHTYPES, "Attempting to resume session...\n");
-      SSL_set_session(mytls_vars->ssl, sess);
+      if (SSL_set_session(mytls_vars->ssl, sess) <= 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "There was an error attempting to resume the session!\n");
+	  tls_funcs_process_error();
+	}
     }
 
   // Set this to SSL_VERIFY_NONE if we don't want to do anything with a failed
@@ -377,7 +322,7 @@ int tls_funcs_parse(struct generic_eap_data *thisint, u_char *indata,
       return XEBADPACKETSIZE;
     }
 
-  if (chunksize == 0)
+  if ((chunksize == 0) || (chunksize > 1398))
     {
       chunksize = 1398;
     }
@@ -393,10 +338,18 @@ int tls_funcs_parse(struct generic_eap_data *thisint, u_char *indata,
     {
       if (indata != NULL)
 	{
-	  BIO_reset(mytls_vars->ssl_in);
+	  if (BIO_reset(mytls_vars->ssl_in) < 1)
+	    {
+	      tls_funcs_process_error();
+	    }
+
 	  BIO_write(mytls_vars->ssl_in, indata, insize);
 	} 
-      BIO_reset(mytls_vars->ssl_out);
+      if (BIO_reset(mytls_vars->ssl_out) < 0)
+	{
+	  tls_funcs_process_error();
+	}
+
       if (mytls_vars->ssl == NULL) 
 	{
 	  debug_printf(DEBUG_NORMAL, "SSL context is NULL!!!!\n");
@@ -404,6 +357,11 @@ int tls_funcs_parse(struct generic_eap_data *thisint, u_char *indata,
 	}
 
       rc = SSL_connect(mytls_vars->ssl);
+      if (rc < 0)
+	{
+	  tls_funcs_process_error();
+	}
+
       BIO_get_mem_ptr(mytls_vars->ssl_out, &retData);
       
       mytls_vars->tlsoutdata = retData->data;
@@ -483,6 +441,9 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
       return XEBADPACKETSIZE;
     }
 
+  debug_printf(DEBUG_AUTHTYPES, "Packet in (%d) :\n", insize);
+  debug_hex_dump(DEBUG_AUTHTYPES, inframe, insize);
+
   // First, make sure we don't have any errors.
   err = ERR_get_error();
   if (err != 0)
@@ -790,7 +751,8 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
   if (SSL_CTX_load_verify_locations(mytls_vars->ctx, root_cert, root_dir) == 0)
     {
       debug_printf(DEBUG_NORMAL, "Failed to initalize path to root certificate!\n");
-      debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
+      tls_funcs_process_error();
+      //      debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
       if(mytls_vars->ctx)
 	{
 	  SSL_CTX_free(mytls_vars->ctx);
@@ -806,7 +768,8 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
     if (SSL_CTX_load_verify_locations(mytls_vars->ctx, NULL, crl_dir) == 0)
       {
 	debug_printf(DEBUG_NORMAL, "Failed to initalize path to CRLs!\n");
-	debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
+	tls_funcs_process_error();
+	//debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
 	if(mytls_vars->ctx)
 	  {
 	    SSL_CTX_free(mytls_vars->ctx);
@@ -821,7 +784,8 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
   if (SSL_CTX_set_default_verify_paths(mytls_vars->ctx) == 0)
     {
       debug_printf(DEBUG_NORMAL, "Failed to initalize default paths for root certificates!\n");
-      debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
+      tls_funcs_process_error();
+      //debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
       if(mytls_vars->ctx)
 	{
 	  SSL_CTX_free(mytls_vars->ctx);
@@ -856,6 +820,7 @@ int tls_funcs_load_random(struct generic_eap_data *thisint, char *random_file)
 
   if (RAND_load_file(file, 1024) < 0)
     {
+      tls_funcs_process_error();
       if(mytls_vars->ctx)
 	{
 	  SSL_CTX_free(mytls_vars->ctx);
@@ -900,6 +865,7 @@ int tls_funcs_load_user_cert(struct generic_eap_data *thisint,
 				   SSL_FILETYPE_PEM) != 1 )
     {
       debug_printf(DEBUG_NORMAL, "Couldn't load client certificate data!\n");
+      tls_funcs_process_error();
       if(mytls_vars->ctx)
 	{
 	  SSL_CTX_free(mytls_vars->ctx);
@@ -915,6 +881,7 @@ int tls_funcs_load_user_cert(struct generic_eap_data *thisint,
       SSL_CTX_use_PrivateKey_file(mytls_vars->ctx, key_file, 
 				  SSL_FILETYPE_ASN1) != 1) 
     {
+      tls_funcs_process_error();
       if(mytls_vars->ctx)
 	{
 	  SSL_CTX_free(mytls_vars->ctx);
@@ -927,6 +894,7 @@ int tls_funcs_load_user_cert(struct generic_eap_data *thisint,
   if (!SSL_CTX_check_private_key(mytls_vars->ctx))
     {
       debug_printf(DEBUG_NORMAL, "Private key isn't valid!\n");
+      tls_funcs_process_error();
       return XETLSCERTLOAD;
     }
 
