@@ -58,48 +58,16 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: ipc_callout.c,v 1.10 2004/04/05 17:19:16 chessing Exp $
- * $Date: 2004/04/05 17:19:16 $
+ * $Id: ipc_callout.c,v 1.12 2004/07/15 04:15:35 chessing Exp $
+ * $Date: 2004/07/15 04:15:35 $
  * $Log: ipc_callout.c,v $
- * Revision 1.10  2004/04/05 17:19:16  chessing
+ * Revision 1.12  2004/07/15 04:15:35  chessing
  *
- * Added additional checks against pointers to try to help prevent segfaults.  (This still needs to be completed.)  Fixed a problem with PEAP where a NULL input packet would result in a huge unencrypted packet, and a segfault.  (This was triggered when using one of the gui password tools.  When the password was in the config file, it wouldn't be triggered.)
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Revision 1.9  2004/03/19 23:43:56  chessing
+ * Revision 1.11  2004/06/15 03:22:29  chessing
  *
- * Lots of changes.  Changed the password prompting code to no longer require the EAP methods to maintain their own stale frame buffer.  (Frame buffer pointers should be moved out of generic_eap_data before a final release.)  Instead, EAP methods should set need_password in generic_eap_data to 1, along with the variables that identify the eap type being used, and the challenge data (if any -- only interesting to OTP/GTC at this point).  Also fixed up xsup_set_pwd.c, and got it back in CVS.  (For some reason, it was in limbo.)  Added xsup_monitor under gui_tools/cli.  xsup_monitor will eventually be a cli program that will monitor XSupplicant (running as a daemon) and display status information, and request passwords when they are not in the config.
- *
- * Revision 1.8  2004/03/17 21:21:40  chessing
- *
- * Hopefully xsup_set_pwd is in the right place now. ;)  Added the functions needed for xsupplicant to request a password from a GUI client.  (Still needs to be tested.)  Updated TTLS and PEAP to support password prompting.  Fixed up curState change in statemachine.c, so it doesn't print [ALL] in front of the current state.
- *
- * Revision 1.7  2004/01/18 06:31:19  chessing
- *
- * A few fixes here and there.  Added support in EAP-TLS to wait for a password to be entered from a "GUI" interface.  Added a small CLI utility to pass the password in to the daemon. (In gui_tools/cli)  Made needed IPC updates/changes to support passing in of a generic password to be used.
- *
- * Revision 1.6  2004/01/14 05:44:48  chessing
- *
- * Added pid file support. (Very basic for now, needs to be improved a little.)  Attempted to add setup of global variables. (Need to figure out why it is segfaulting.)  Added more groundwork for IPC.
- *
- * Revision 1.5  2004/01/06 23:35:07  chessing
- *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.4  2003/12/23 04:57:10  chessing
- *
- * IPC additions, GUI client routines.
- *
- * Revision 1.3  2003/12/19 06:29:56  chessing
- *
- * New code to determine if an interface is wireless or not.  Lots of IPC updates.
- *
- * Revision 1.2  2003/12/18 02:09:45  chessing
- *
- * Some small fixes, and working IPC code to get interface state.
- *
- * Revision 1.1  2003/12/14 06:17:09  chessing
- *
- * Added ipc_callout.* needed to build.
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -212,7 +180,7 @@ void ipc_callout_process_conf(struct interface_data *thisint, int *bufptr,
  * Register a client to receive pushed messages from the daemon.
  *
  ****************************************************************/
-void ipc_callout_reg_client(struct interface_data *thisint, int *bufptr,
+void ipc_callout_reg_client(struct ipc_struct *ipc, int *bufptr,
 			    char *buffer, int bufsize, char *resbuf, 
 			    int *resbufptr, struct sockaddr *mysock)
 {
@@ -220,8 +188,7 @@ void ipc_callout_reg_client(struct interface_data *thisint, int *bufptr,
   struct registered_clients *cur;
   struct ipc_header *header;
 
-  if ((!thisint) || (!bufptr) || (!buffer) || (!resbuf) || (!resbufptr) ||
-      (!mysock) || (!thisint->ipc))
+  if ((!bufptr) || (!buffer) || (!resbuf) || (!resbufptr) || (!mysock))
     {
       debug_printf(DEBUG_NORMAL, "Invalid data passed in to ipc_callout_reg_client()!\n");
       return;
@@ -252,12 +219,12 @@ void ipc_callout_reg_client(struct interface_data *thisint, int *bufptr,
 
   // Now register the client.  And, assuming we register correctly, return
   // an ACK.
-  if (thisint->ipc->reged == NULL)
+  if (ipc->reged == NULL)
     {
-      thisint->ipc->reged = (struct registered_clients *)malloc(sizeof(struct registered_clients));
-      cur = thisint->ipc->reged;
+      ipc->reged = (struct registered_clients *)malloc(sizeof(struct registered_clients));
+      cur = ipc->reged;
     } else {
-      cur = thisint->ipc->reged;
+      cur = ipc->reged;
 
       while (cur->next != NULL)
 	{
@@ -548,14 +515,13 @@ void ipc_callout_set_password(struct interface_data *thisint, int *bufptr,
  * challenge string that the EAP type may need.
  *
  ***********************************************************************/
-void ipc_callout_request_password(struct interface_data *thisint, 
-				  int *bufptr, char *buffer, int bufsize,
+void ipc_callout_request_password(int *bufptr, char *buffer, int bufsize,
 				  char *eapname, char *challenge)
 {
   struct ipc_cmd *cmd;
   struct ipc_header *header;
 
-  if ((!thisint) || (!bufptr) || (!buffer) || (!eapname))
+  if ((!bufptr) || (!buffer) || (!eapname))
     {
       debug_printf(DEBUG_NORMAL, "Invalid data passed in to ipc_callout_request_password()!\n");
       return;
