@@ -59,73 +59,20 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: statemachine.c,v 1.17 2004/04/05 17:19:16 chessing Exp $
- * $Date: 2004/04/05 17:19:16 $
+ * $Id: statemachine.c,v 1.20 2004/08/10 01:59:26 chessing Exp $
+ * $Date: 2004/08/10 01:59:26 $
  * $Log: statemachine.c,v $
- * Revision 1.17  2004/04/05 17:19:16  chessing
+ * Revision 1.20  2004/08/10 01:59:26  chessing
  *
- * Added additional checks against pointers to try to help prevent segfaults.  (This still needs to be completed.)  Fixed a problem with PEAP where a NULL input packet would result in a huge unencrypted packet, and a segfault.  (This was triggered when using one of the gui password tools.  When the password was in the config file, it wouldn't be triggered.)
+ * Added support for the SNMP supplicant counters defined in the IEEE 802.1X-2001 document.
  *
- * Revision 1.16  2004/03/29 21:57:13  chessing
+ * Revision 1.19  2004/07/15 04:15:35  chessing
  *
- * Changed the socket number we use for communication with the daemon from 10240 (which seems like a bad choice) to 26798 (which seems a little more random ;).  Also changed our debug code so that it doesn't output to the console when we are running in daemon mode.  The only way to get debug info while in daemon mode is to set a log file!!!
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Revision 1.15  2004/03/27 01:40:45  chessing
- *
- * Lots of small updates to free memory that wasn't getting freed, add some additional debug output, and fix a couple of memory leaks.
- *
- * Revision 1.14  2004/03/25 06:06:56  chessing
- *
- * Some debug code cleanups.  Fixed a bug with non-existant, or down interfaces defined in the allow_interfaces would loop forever.  Added calls to reset wireless keys to all 0s when we end up in disconnected, or held state.
- *
- * Revision 1.13  2004/03/24 18:35:46  chessing
- *
- * Added a modified version of a patch from David Relson to fix a problem with some of the debug info in config_grammer.y.  Added some additional checks to eapol_key_type1 that will keep us from segfaulting under some *REALLY* strange conditions.  Changed the set key code in cardif_linux to double check that we aren't a wireless interface before returning an error.  This resolved a problem when XSupplicant was started when an interface was done.  Upon bringing up the interface, XSupplicant would sometimes think it wasn't wireless, and not bother trying to set keys.
- *
- * Revision 1.12  2004/03/22 00:41:00  chessing
- *
- * Added logfile option to the global config options in the config file.  The logfile is where output will go when we are running in daemon mode.  If no logfile is defined, output will go to the console that started xsupplicant.   Added forking to the code, so that when started, the process can daemonize, and run in the background.  If there is a desire to force running in the foreground (such as for debugging), the -f option was added.
- *
- * Revision 1.11  2004/03/17 21:21:40  chessing
- *
- * Hopefully xsup_set_pwd is in the right place now. ;)  Added the functions needed for xsupplicant to request a password from a GUI client.  (Still needs to be tested.)  Updated TTLS and PEAP to support password prompting.  Fixed up curState change in statemachine.c, so it doesn't print [ALL] in front of the current state.
- *
- * Revision 1.10  2004/03/06 03:53:54  chessing
- *
- * We now send logoffs when the process is terminated.  Added a new option to the config file "wireless_control" which will allow a user to disable non-EAPoL key changes.  Added an update to destination BSSID checking that will reset the wireless key to all 0s when the BSSID changes.  (This is what "wireless_control" disables when it is set to no.)  Roaming should now work, but because we are resetting keys to 128 bit, there may be issues with APs that use 64 bit keys.  I will test this weekend.
- *
- * Revision 1.9  2004/02/09 21:50:06  chessing
- *
- * Added patches from Jouni Malinen.  Includes an EAP-MD5 fix, dec_if_nz() patch, more debugging information from the state machine, and global state changes.
- *
- * Revision 1.8  2004/01/18 06:31:19  chessing
- *
- * A few fixes here and there.  Added support in EAP-TLS to wait for a password to be entered from a "GUI" interface.  Added a small CLI utility to pass the password in to the daemon. (In gui_tools/cli)  Made needed IPC updates/changes to support passing in of a generic password to be used.
- *
- * Revision 1.7  2004/01/15 23:45:10  chessing
- *
- * Fixed a segfault when looking for wireless interfaces when all we had was a wired interface.  Fixed external command execution so that junk doesn't end up in the processed string anymore.  Changed the state machine to call txRspAuth even if there isn't a frame to process.  This will enable EAP methods to request information from a GUI interface (such as passwords, or supply challenge information that might be needed to generate passwords).  EAP methods now must decide what to do when they are handed NULL for the pointer to the in frame.  If they don't need any more data, they should quietly exit.
- *
- * Revision 1.6  2004/01/15 01:12:44  chessing
- *
- * Fixed a keying problem (keying material wasn't being generated correctly).  Added support for global counter variables from the config file. (Such as auth_period)  Added support for executing command defined in the config file based on different events.  (Things such as what to do on reauth.)  Added the ability to roam to a different SSID.  We now check to make sure our BSSID hasn't changed, and we follow it, if it has.  Fixed a sefault when the program was terminated in certain states.  Added attempt at better garbage collection on program termination. Various small code cleanups.
- *
- * Revision 1.5  2003/11/29 03:50:03  chessing
- *
- * Added NAK code, EAP Type checking, split out daemon config from user config, added Display of EAP-Notification text, revamped phase 2 selection method for TTLS.
- *
- * Revision 1.4  2003/11/28 07:46:23  chessing
- *
- * EAPOL no longer uses malloc for allocating the frame buffers.  State machine init stuff is now in statemachine.c where it belongs.  eap_init() now accepts an interface_data struct, to make it conform to other init calls.
- *
- * Revision 1.3  2003/11/24 04:56:03  chessing
- *
- * EAP-SIM draft 11 now works.  Statemachine updated to work based on the up/down state of an interface, rather than just assuming it is up.
- *
- * Revision 1.2  2003/11/19 04:23:18  chessing
- *
- * Updates to fix the import
+ * Revision 1.18  2004/06/15 03:22:29  chessing
  *
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -133,6 +80,7 @@
 #include <stdio.h>
 #include <netinet/in.h>
 
+#include "snmp.h"
 #include "statemachine.h"
 #include "xsup_debug.h"
 #include "frame_structs.h"
@@ -141,6 +89,7 @@
 #include "eapol.h"
 #include "xsup_err.h"
 #include "cardif/cardif.h"
+#include "eap_types/leap/eapleap.h"
 
 /******************************************
  *
@@ -278,7 +227,7 @@ int statemachine_run(struct interface_data *thisint, char *inframe,
       debug_printf(DEBUG_STATE, "(global) -> DISCONNECTED\n");
       thisint->statemachine->curState = DISCONNECTED;
       thisint->statemachine->initialize = FALSE;
-      if (thisint->isWireless == TRUE) cardif_reset_keys(thisint);
+      if (thisint->flags & IS_WIRELESS) cardif_reset_keys(thisint);
     }
   else if (thisint->statemachine->eapFail &&
 	   !(thisint->statemachine->initialize || 
@@ -289,7 +238,7 @@ int statemachine_run(struct interface_data *thisint, char *inframe,
       debug_printf(DEBUG_STATE, "(global) -> HELD\n");
       thisint->statemachine->lastState = thisint->statemachine->curState;
       thisint->statemachine->curState = HELD;
-      if (thisint->isWireless == TRUE) cardif_reset_keys(thisint);
+      if (thisint->flags & IS_WIRELESS) cardif_reset_keys(thisint);
     }
   else if (thisint->statemachine->userLogoff &&
 	   !thisint->statemachine->logoffSent &&
@@ -299,7 +248,7 @@ int statemachine_run(struct interface_data *thisint, char *inframe,
       debug_printf(DEBUG_STATE, "(global) -> LOGOFF\n");
       thisint->statemachine->lastState = thisint->statemachine->curState;
       thisint->statemachine->curState = LOGOFF;
-      if (thisint->isWireless == TRUE) cardif_reset_keys(thisint);
+      if (thisint->flags & IS_WIRELESS) cardif_reset_keys(thisint);
     }
   else if (thisint->statemachine->eapSuccess &&
 	   !(thisint->statemachine->initialize || 
@@ -336,6 +285,7 @@ int statemachine_run(struct interface_data *thisint, char *inframe,
 	    (thisint->statemachine->portEnabled == FALSE)))
 	{
 	  debug_printf(DEBUG_STATE, "Processing LOGOFF state.\n");
+	  thisint->snmp->dot1xSuppEapolLogoffFramesTx++;
 	  txLogoff(outframe, outsize);
 	  thisint->statemachine->logoffSent = TRUE;
 	  thisint->statemachine->suppStatus = UNAUTHORIZED;
@@ -469,6 +419,7 @@ int statemachine_run(struct interface_data *thisint, char *inframe,
 	  thisint->statemachine->startWhen = thisint->statemachine->startPeriod;
 	  thisint->statemachine->startCount++;
 	  thisint->statemachine->reqId = FALSE;
+	  thisint->snmp->dot1xSuppEapolStartFramesTx++;
 	  txStart(outframe, outsize);
 	  retVal = XDATA;
 	}
@@ -554,11 +505,12 @@ int txLogoff(char *outframe, int *outsize)
 int txRspId(struct interface_data *thisint, char *outframe, int *outsize)
 {
   struct eapol_header *myframe;
-  int eapsize, outfsize;
+  int eapsize;
 
   debug_printf(DEBUG_STATE, "Sending EAPOL-Response-Identification\n");
 
-  if ((!thisint) || (!outframe) || (!outsize))
+  if ((!thisint) || (!outframe) || (!outsize) || (!thisint->userdata) ||
+      (!thisint->statemachine))
     {
       debug_printf(DEBUG_NORMAL, "Invalid data passed in to txRspId()!\n");
       return XEMALLOC;
@@ -570,11 +522,19 @@ int txRspId(struct interface_data *thisint, char *outframe, int *outsize)
   myframe->eapol_version = MAX_EAPOL_VER;
   myframe->eapol_type = EAP_PACKET;
 
-  eap_request_id(thisint, outframe, &outfsize, &eapsize);
+  // See if we need to populate our ID.
+  eap_prepopulate_id(thisint);
+
+  // Now, build the response ID packet.
+  eap_request_id(thisint->userdata->identity, 
+		 thisint->statemachine->receivedId, 
+		 (char *)&outframe[OFFSET_TO_EAP], &eapsize);
 
   myframe->eapol_length = htons(eapsize); 
 
-  *outsize = (outfsize + OFFSET_PAST_MAC);
+  *outsize = (eapsize + OFFSET_TO_EAP);
+
+  thisint->snmp->dot1xSuppEapolRespIdFramesTx++;
 
   return XDATA;
 }
@@ -589,7 +549,8 @@ int txRspAuth(struct interface_data *thisint, char *inframe, int insize,
 	      char *outframe, int *outsize)
 {
   struct eapol_header *myframe;
-  int payloadsize, eapsize=0;
+  int eapsize=0;
+  char *tempframe;
 
   debug_printf(DEBUG_STATE, "Sending EAPOL-Response-Authentication\n");
 
@@ -607,7 +568,30 @@ int txRspAuth(struct interface_data *thisint, char *inframe, int insize,
 
   myframe = (struct eapol_header *)&outframe[OFFSET_PAST_MAC];
 
-  eap_request_auth(thisint, inframe, insize, outframe, &payloadsize, &eapsize);
+  if (inframe == NULL)
+    {
+      tempframe = NULL;
+    } else {
+      tempframe = (char *)&inframe[OFFSET_TO_EAP];
+    }
+
+  if (eap_create_active_method(&thisint->userdata->activemethod,
+			       thisint->userdata->identity,
+			       thisint->tempPassword) != 0)
+    {
+      debug_printf(DEBUG_NORMAL, "Couldn't build active method!  Authentication will not happen!\n");
+      return XEMALLOC;
+    }
+
+  if (eap_request_auth(thisint->userdata->activemethod, 
+		       thisint->userdata->methods, tempframe, insize, 
+		       (char *)&outframe[OFFSET_TO_EAP], &eapsize) == 2)
+    {
+      // We got a LEAP success, so we need to tell the state machine. And
+      // create the keying material.
+      thisint->statemachine->eapSuccess = 1;
+      eapleap_get_keys(thisint);
+    }
 
   if (eapsize != 0)
     {
@@ -616,10 +600,13 @@ int txRspAuth(struct interface_data *thisint, char *inframe, int insize,
       myframe->eapol_type = EAP_PACKET;
       myframe->eapol_length = htons(eapsize);
 
-      *outsize = (payloadsize + OFFSET_PAST_MAC);
+      //      *outsize = (payloadsize + OFFSET_PAST_MAC);
+      *outsize = (eapsize + OFFSET_TO_EAP);
     } else {
       *outsize = 0;
     }
+
+  thisint->snmp->dot1xSuppEapolRespFramesTx++;
   return *outsize;
 }
 
