@@ -59,173 +59,57 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: xsup_driver.c,v 1.39 2004/05/17 02:45:45 chessing Exp $
- * $Date: 2004/05/17 02:45:45 $
+ * $Id: xsup_driver.c,v 1.52 2004/08/15 04:29:34 chessing Exp $
+ * $Date: 2004/08/15 04:29:34 $
  * $Log: xsup_driver.c,v $
- * Revision 1.39  2004/05/17 02:45:45  chessing
+ * Revision 1.52  2004/08/15 04:29:34  chessing
  *
- * Small cleanups prior to a final release.  (No bugs, or anything interesting.)
+ * Completed support for scanning and joining wireless networks.  If your interface isn't configured, we will scan to gather information about all known wireless networks.  If a network in the list has a configuration, we will join it.  (Including turning on needed encryption.)  There are still a few rough edges that need to be hammered out.  But the overall user experience should be improved.
  *
- * Revision 1.38  2004/04/29 17:51:08  chessing
+ * Revision 1.51  2004/08/13 04:20:00  chessing
  *
- * Fix a logic error in xsup_debug.c.  Not defining a log file no longer segfaults.
+ * Accepted fork() patch from i3keba.  (Slightly modified version of patch.)
  *
- * Revision 1.37  2004/04/26 20:51:12  chessing
+ * Revision 1.50  2004/08/11 04:01:17  chessing
  *
- * Patch to attempt to fix the init_interface_* errors reported on the list.  Removed password clearing on failed authentication attempts.  Password clearing currently has some issues that will prevent it from being in the 1.0 stable.
+ * The -d option now supports letter designations for each debug level, in addition to 0..7.  (However, both letters and numbers can't be used at the same time!)  Also changed some formatting for the SNMP scoreboard.
  *
- * Revision 1.36  2004/04/06 20:31:26  chessing
+ * Revision 1.49  2004/08/05 23:56:18  chessing
  *
- * PEAP NOW WORKS WITH IAS!!!!!! (Thanks to help from Matthew Gast!! (We love you! ;))  Also, added patches from yesterday's testing at iLabs, including some keying fixes, some segfault fixes, and a few other misc. issues.  iLabs testing has been worth it!
+ * Added basic support for scanning for broadcast SSIDs.  This is another step closer to WPA/11i support. ;)
  *
- * Revision 1.35  2004/04/05 17:19:16  chessing
+ * Revision 1.48  2004/07/25 19:36:40  chessing
+ * Fixed a few more logical/bitwise nots.  Added rtnetlink support for hot-plugging interfaces.  (Hot unplugging is coming soon. ;)
  *
- * Added additional checks against pointers to try to help prevent segfaults.  (This still needs to be completed.)  Fixed a problem with PEAP where a NULL input packet would result in a huge unencrypted packet, and a segfault.  (This was triggered when using one of the gui password tools.  When the password was in the config file, it wouldn't be triggered.)
+ * Revision 1.47  2004/07/23 04:05:49  chessing
+ * Fixed a segfault problem.  Started to add rtnetlink support.
  *
- * Revision 1.34  2004/03/29 21:57:13  chessing
+ * Revision 1.46  2004/07/23 02:14:38  chessing
+ * Moved main event loop from xsup_driver.c in to an OS specific file to allow for easier porting.
  *
- * Changed the socket number we use for communication with the daemon from 10240 (which seems like a bad choice) to 26798 (which seems a little more random ;).  Also changed our debug code so that it doesn't output to the console when we are running in daemon mode.  The only way to get debug info while in daemon mode is to set a log file!!!
+ * Revision 1.45  2004/07/21 22:47:53  chessing
  *
- * Revision 1.33  2004/03/27 01:40:45  chessing
+ * Fixed bitwise not problem identified in bugid 995523.
  *
- * Lots of small updates to free memory that wasn't getting freed, add some additional debug output, and fix a couple of memory leaks.
+ * Revision 1.44  2004/07/19 02:43:16  chessing
  *
- * Revision 1.32  2004/03/26 21:34:51  chessing
- * Fixed problem with interface being down on startup causing xsupplicant to not read the proper configuration information when the interface is brought up.  Added/fixed code to rebuild userdata piece of structure when the essid changes.  Added code to avoid setting a key on an interface if the interface doesn't already have encryption enabled.  Added a little bit of debugging code to help find a solution to an IPC socket problem.
+ * Changed things to get rid of Linux specific pieces in the interface_data struct. Fixed up EAP-SIM and EAP-AKA to skip AVPs that we don't support.  (We print a mesage, and move on.)  Added the --enable-radiator-test option to configure EAP-AKA to use the test vectors included with Radiator's AKA module.  (To use this mode, no SIM card is required, but the PCSC library is still needed to build.  Also, some errors will be displayed.)
  *
- * Revision 1.31  2004/03/25 06:06:56  chessing
+ * Revision 1.43  2004/07/15 04:15:35  chessing
  *
- * Some debug code cleanups.  Fixed a bug with non-existant, or down interfaces defined in the allow_interfaces would loop forever.  Added calls to reset wireless keys to all 0s when we end up in disconnected, or held state.
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Revision 1.30  2004/03/24 08:21:01  galimorerpg
- * Added Pavel Roskin's "no_int" patch, which prevents xsupplicant from crashing when cleaning up on a system where no valid interfaces are present.
+ * Revision 1.42  2004/07/01 01:27:44  chessing
  *
- * Revision 1.29  2004/03/24 08:16:14  galimorerpg
- * Added Pavel Roskin's deny_first patch:
+ * Added some patches submitted to sourceforge by users.  Including openssl static/dynamic issue, and zombie children.
  *
- * If I put an interface to the deny list it means that I don't want xsupplicant to touch it in any way.  In particular, it should not be probed and validated, whatever it means.
+ * Revision 1.41  2004/06/21 05:19:12  chessing
  *
- * The attached patch swaps the order of the checks - deny list is checked before cardif_validate()
+ * Added a few minor fixes to EAP-AKA support.  Added "allmulti" as a global configuration option.  (By default, allmulti is now on!)
  *
+ * Revision 1.40  2004/06/15 03:22:29  chessing
  *
- * A small typo fix was also added to xsup_driver.c
- *
- * Revision 1.28  2004/03/24 07:42:33  galimorerpg
- * Fixed a *NASTY* recursive loop in config.c/config_get_logfile()
- * Where config_get_logfile was debug_printf()ing and debug_printf()
- * was calling config_get_logfile.  I've worked around this by *NOT*
- * debug_printf()ing from config_get_logfile()
- *
- * On a side note, I found this bug because xsupplicant was crashing.
- * There's a handy option in the 2.4.25 kernel (CONFIG_OOM_KILLER) that
- * can kill programs that it decides are trying to use too much memory.
- *
- * Interestingly enough, valgrind didn't crash on this problem, presumably
- * because of its memory management?  Either that or the kernel didn't think
- * valgrind was hitting the memory hard enough.
- *
- * Revision 1.27  2004/03/22 00:41:00  chessing
- *
- * Added logfile option to the global config options in the config file.  The logfile is where output will go when we are running in daemon mode.  If no logfile is defined, output will go to the console that started xsupplicant.   Added forking to the code, so that when started, the process can daemonize, and run in the background.  If there is a desire to force running in the foreground (such as for debugging), the -f option was added.
- *
- * Revision 1.26  2004/03/20 05:38:38  chessing
- *
- * PID file fixed.  We now store the correct pid in the /var/run/xsupplicant pid file.
- *
- * Revision 1.25  2004/03/06 03:53:54  chessing
- *
- * We now send logoffs when the process is terminated.  Added a new option to the config file "wireless_control" which will allow a user to disable non-EAPoL key changes.  Added an update to destination BSSID checking that will reset the wireless key to all 0s when the BSSID changes.  (This is what "wireless_control" disables when it is set to no.)  Roaming should now work, but because we are resetting keys to 128 bit, there may be issues with APs that use 64 bit keys.  I will test this weekend.
- *
- * Revision 1.24  2004/02/28 01:26:38  chessing
- *
- * Several critical updates.  Fixed the HMAC failure on some keys. (This was due to a lot more than just an off-by-one.)  Fixed up the key decryption routine to identify key packets with no encrypted key, and use the peer key instead.  When using the peer key, we also can handle packets that are padded funny.  (Our Cisco AP1200 has two null pad bytes at the end of some key frames.)  Changed the response ID function to not add a 00 to the end of the ID.  The 00 byte shouldn't have been seen by the RADIUS server unless they were not paying attention to the EAP-Length.  So, this wasn't really a bug fix.  Started to add support for CN checking for TLS based protocols.
- *
- * Revision 1.23  2004/02/16 05:05:59  chessing
- *
- * Added support for the deny_interfaces, and allow_interfaces options in the config file.  (We should now have support for *EVERYTHING* in the new config file format!)  Updated EAP types other than SIM to use the new password prompt/delay code.  Phase 2 of TTLS still needs to be completed, along with the code that actually notifies the GUI.
- *
- * Revision 1.22  2004/02/13 05:51:32  chessing
- *
- * Removed pieces from sha1.c that were duplicates for OpenSSL calls.  Hopefully this will resolve the TLS issues that have been under discussion on the list.  Added support for a default path for the config file.  If a config file is not specified on the command line, xsupplicant will attempt to read it from /etc/xsupplicant.conf.  Moved code to request a password from each of the EAP types to interface.c/h.  Currently this change is only implemented in the EAP-SIM module.  The changes to the GUI prompt code now make more sense, and are easier to follow.  It will be updated in other EAP types soon.
- *
- * Revision 1.21  2004/02/07 07:19:37  chessing
- *
- * Fixed EAP-SIM so that it works with FreeRADIUS correctly.  Fixed a bunch of memory leaks in the EAP-SIM, and related code.
- *
- * Revision 1.20  2004/02/06 06:13:31  chessing
- *
- * Cleaned up some unneeded stuff in the configure.in file as per e-mail from Rakesh Patel.  Added all 12 patches from Jouni Malinen (Including wpa_supplicant patch, until we can add true wpa support in xsupplicant.)
- *
- * Revision 1.19  2004/01/18 06:31:19  chessing
- *
- * A few fixes here and there.  Added support in EAP-TLS to wait for a password to be entered from a "GUI" interface.  Added a small CLI utility to pass the password in to the daemon. (In gui_tools/cli)  Made needed IPC updates/changes to support passing in of a generic password to be used.
- *
- * Revision 1.18  2004/01/17 21:16:15  chessing
- *
- * Various segfault fixes.  PEAP now works correctly again.  Some new error checking in the tls handlers.  Fixes for the way we determine if we have changed ESSIDs.  We now quit when we don't have a config, or when the config is bad. Added code to check and see if a frame is in the queue, and don't sleep if there is.  "Fixed" ID issue by inheriting the ID from the parent where needed.  However, assigning an ID inside of a handler will override the parent ID.  This could cause problems with some EAP types.  We should add a "username" field to PEAP to allow configuration of the inner EAP identity.
- *
- * Revision 1.17  2004/01/15 23:45:10  chessing
- *
- * Fixed a segfault when looking for wireless interfaces when all we had was a wired interface.  Fixed external command execution so that junk doesn't end up in the processed string anymore.  Changed the state machine to call txRspAuth even if there isn't a frame to process.  This will enable EAP methods to request information from a GUI interface (such as passwords, or supply challenge information that might be needed to generate passwords).  EAP methods now must decide what to do when they are handed NULL for the pointer to the in frame.  If they don't need any more data, they should quietly exit.
- *
- * Revision 1.16  2004/01/15 01:12:44  chessing
- *
- * Fixed a keying problem (keying material wasn't being generated correctly).  Added support for global counter variables from the config file. (Such as auth_period)  Added support for executing command defined in the config file based on different events.  (Things such as what to do on reauth.)  Added the ability to roam to a different SSID.  We now check to make sure our BSSID hasn't changed, and we follow it, if it has.  Fixed a sefault when the program was terminated in certain states.  Added attempt at better garbage collection on program termination. Various small code cleanups.
- *
- * Revision 1.15  2004/01/14 22:07:25  chessing
- *
- * Fixes that were needed in order to allow us to authenticate correctly.  We should now be able to authenticate using only information provided by the config file!
- *
- * Revision 1.14  2004/01/14 05:44:48  chessing
- *
- * Added pid file support. (Very basic for now, needs to be improved a little.)  Attempted to add setup of global variables. (Need to figure out why it is segfaulting.)  Added more groundwork for IPC.
- *
- * Revision 1.13  2004/01/13 01:55:55  chessing
- *
- * Major changes to EAP related code.  We no longer pass in an interface_data struct to EAP handlers.  Instead, we hand in a generic_eap_data struct which containsnon-interface specific information.  This will allow EAP types to be reused as phase 2 type easier.  However, this new code may create issues with EAP types that make use of the identity in the eap type.  Somehow, the identity value needs to propigate down to the EAP method.  It currently does not.  This should be any easy fix, but more testing will be needed.
- *
- * Revision 1.12  2004/01/06 23:35:07  chessing
- *
- * Fixed a couple known bugs in SIM.  Config file support should now be in place!!! But, because of the changes, PEAP is probably broken.  We will need to reconsider how the phase 2 piece of PEAP works.
- *
- * Revision 1.11  2003/12/28 07:13:21  chessing
- *
- * Fixed a problem where we would segfault on an EAP type we didn't understand.  Added EAP-OTP.  EAP-OTP has been tested using the opie package, and Radiator 3.8.  EAP-OTP currently prompts for a passphrase, which it shouldn't do, so it should be considered *VERY* much in test mode until we finish the GUI.
- *
- * Revision 1.10  2003/12/19 06:29:56  chessing
- *
- * New code to determine if an interface is wireless or not.  Lots of IPC updates.
- *
- * Revision 1.9  2003/12/18 02:09:45  chessing
- *
- * Some small fixes, and working IPC code to get interface state.
- *
- * Revision 1.8  2003/12/14 06:11:03  chessing
- *
- * Fixed some stuff with SIM in relation to the new config structures.  Cleaned out CR/LF from LEAP source files.  Added user certificate support to TTLS and PEAP. Some additions to the IPC code. (Not tested yet.)
- *
- * Revision 1.7  2003/12/10 14:33:16  npetroni
- * removed segfault that occurs when calling program without an interface.
- *
- * Revision 1.6  2003/12/07 06:20:19  chessing
- *
- * Changes to deal with new config file style.  Beginning of IPC code.
- *
- * Revision 1.5  2003/12/04 04:36:24  chessing
- *
- * Added support for multiple interfaces (-D now works), also added DEBUG_EXCESSIVE to help clean up some of the debug output (-d 6).
- *
- * Revision 1.4  2003/11/29 03:50:03  chessing
- *
- * Added NAK code, EAP Type checking, split out daemon config from user config, added Display of EAP-Notification text, revamped phase 2 selection method for TTLS.
- *
- * Revision 1.3  2003/11/24 12:13:54  npetroni
- * Added catch for bad arguments
- *
- * Revision 1.2  2003/11/19 04:23:18  chessing
- *
- * Updates to fix the import
- *
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -241,6 +125,7 @@
 #include <signal.h>
 #include <strings.h>
 #include <string.h>
+#include <sys/wait.h>
 
 #include "profile.h"
 #include "config.h"
@@ -252,8 +137,7 @@
 #include "eapol.h"
 #include "cmd_handler.h"
 #include "cardif/cardif.h"
-
-#define XVERSION "1.0pre2"
+#include "cardif/core.h"
 
 int wpa_keying = 0;
 char pidfileName[25] = "/var/run/xsupplicant";
@@ -328,42 +212,13 @@ void alrmclock()
   while (intcur != NULL)
     {
       intcur->statemachine->tick = TRUE;
+      event_core_clock_tick(intcur);
       intcur = intcur->next;
     }
+
   alarm(1);
 }
 
-/*********************************************
- *
- * Determine if the interface given is in the list of interfaces.
- *
- *********************************************/
-int int_in_list(char *intName, struct config_string_list *strList)
-{
-  struct config_string_list *cur;
-
-  if ((!intName) || (!strList))
-    {
-      debug_printf(DEBUG_NORMAL, "Invalid data passed in to int_in_list()!\n");
-      return XEMALLOC;
-    }
-
-  // Our check is case sensative!
-  cur = strList;
-
-  while ((cur != NULL) && (strcmp(intName, cur->name) != 0))
-    {
-      cur = cur->next;
-    }
-
-  if (cur == NULL)
-    {
-      return FALSE;    // The interface isn't in our list.
-    } 
-
-  return TRUE;
-}
-
 /*********************************************
  *
  * Add the interfaces that are in our allow list, that weren't already
@@ -375,13 +230,12 @@ int int_in_list(char *intName, struct config_string_list *strList)
  * require manual definition in the config file.
  *
  *********************************************/
-void add_other_allowed_interfaces(struct interface_data **myint,
-				  struct daemon_conf *dcfg)
+void add_other_allowed_interfaces(struct interface_data **myint)
 {
   struct config_string_list *allowints, *icur;
   struct interface_data *cur;
 
-  if ((!myint) || (!dcfg))
+  if (!myint)
     {
       return;
     }
@@ -407,7 +261,7 @@ void add_other_allowed_interfaces(struct interface_data **myint,
       cur = *myint;
       memset(cur, 0, sizeof(struct interface_data));
 	  
-      if (init_interface_struct(cur, icur->name, dcfg) != XENONE)
+      if (init_interface_struct(cur, icur->name) != XENONE)
 	{
 	  debug_printf(DEBUG_NORMAL, "Couldn't initialize the interface struct!\n");
 	  return;
@@ -416,7 +270,8 @@ void add_other_allowed_interfaces(struct interface_data **myint,
       cardif_init(cur);
       debug_printf(DEBUG_NORMAL, "Interface %s initalized!\n",cur->intName);
 
-      cur->isWireless = FALSE;
+      //      cur->flags = (cur->flags & (~IS_WIRELESS));
+      UNSET_FLAG(cur->flags, IS_WIRELESS);
 
       cur->userdata = config_build(cur->cur_essid);
       config_set_globals(cur);
@@ -450,7 +305,7 @@ void add_other_allowed_interfaces(struct interface_data **myint,
 	  cur = cur->next;
 	  memset(cur, 0, sizeof(struct interface_data));
 	  
-	  if (init_interface_struct(cur, icur->name, dcfg) != XENONE)
+	  if (init_interface_struct(cur, icur->name) != XENONE)
 	    {
 	      debug_printf(DEBUG_NORMAL, "Couldn't init interface struct for interface %s! Cannot continue!\n", icur->name);
 	    }
@@ -458,7 +313,8 @@ void add_other_allowed_interfaces(struct interface_data **myint,
 	  cardif_init(cur);
 	  debug_printf(DEBUG_NORMAL, "Interface %s initalized!\n",cur->intName);
       
-	  cur->isWireless = FALSE;
+	  //          cur->flags = (cur->flags & (~IS_WIRELESS));
+	  UNSET_FLAG(cur->flags, IS_WIRELESS);
 
 	  cur->userdata = config_build(cur->cur_essid);
 	  config_set_globals(cur);
@@ -478,8 +334,7 @@ void add_other_allowed_interfaces(struct interface_data **myint,
  * THIS FUNCTION SHOULD NEVER BE CALLED OUTSIDE OF THIS FILE!
  *
  *********************************************/
-int global_init(int new_debug, int xdaemon, char *device, char *config,
-		struct daemon_conf **dcfg)
+int global_init(int xdaemon, char *device, char *config)
 {
   struct interface_data *intcur;
   struct config_string_list *denyints;
@@ -487,17 +342,11 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
   char intName[16];
   char *default_cfg = "/etc/xsupplicant.conf";
 
-  if (!dcfg)
-    {
-      debug_printf(DEBUG_NORMAL, "Invalid data passed in to global_init()!\n");
-      return XEMALLOC;
-    }
-
   // Set our debug level.
   int_list = NULL;
   intcur = NULL;
 
-  debug_setlevel(new_debug, xdaemon);
+  debug_setdaemon(xdaemon);
 
   if (config == NULL) 
     {
@@ -523,6 +372,9 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
   // Get a list of interfaces we allow, and deny.
   denyints = config_denied_interfaces();
 
+  // Initalize the event core.
+  event_core_setup();
+
   if (xdaemon > 0)
     {
       // If we are a daemon, we ignore any passed in information,
@@ -539,7 +391,7 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
 	    {
 	      // Check to see if this interface is in our list of 
 	      //  interfaces to ignore.
-	      denyint = int_in_list(intName, denyints);
+	      denyint = config_int_in_list(intName, denyints);
 	    } else {
 	      debug_printf(DEBUG_CONFIG, "List of denied interfaces is empty! All interfaces will be used!\n");
 	    }
@@ -555,7 +407,7 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
 		      if (int_list == NULL) return XEMALLOC;
 		      memset(int_list, 0, sizeof(struct interface_data));
 		      
-		      if (init_interface_struct(int_list, (char *)&intName, *dcfg) != XENONE)
+		      if (init_interface_struct(int_list, (char *)&intName) != XENONE)
 			{
 			  debug_printf(DEBUG_NORMAL, "Couldn't init interface struct %s!!!! Cannot continue!\n");
 			  exit(1);
@@ -570,7 +422,7 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
 		      memset(intcur->next, 0, sizeof(struct interface_data));
 		      
 		      intcur = intcur->next;
-		      if (init_interface_struct(intcur, (char *)&intName, *dcfg) != XENONE)
+		      if (init_interface_struct(intcur, (char *)&intName) != XENONE)
 			{
 			  debug_printf(DEBUG_NORMAL, "Couldn't init interface stuct for interface %s!!  Cannot continue!\n", intName);
 			  exit(1);
@@ -584,18 +436,16 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
 		    } else {
 		      debug_printf(DEBUG_NORMAL, "Interface %s initalized!\n",intcur->intName);
 
-		      // If we don't know what kind of interface this is....
-		      if (intcur->isWireless == -1)
+		      // Check to see if this interface is wireless.
+		      if (cardif_int_is_wireless(intcur->intName) == TRUE)
 			{
-			  // Check to see if this interface is wireless.
-			  if (cardif_int_is_wireless(intcur->intName) == TRUE)
-			    {
-			      intcur->isWireless = TRUE;
-			      cardif_check_dest(intcur);
-			    } else {
-			      intcur->isWireless = FALSE;
-			    }
+			  SET_FLAG(intcur->flags, IS_WIRELESS);
+			  cardif_check_dest(intcur);
+			  cardif_start_wireless_scan(intcur);
+			} else {
+			  UNSET_FLAG(intcur->flags, IS_WIRELESS);
 			}
+
 		      intcur->userdata = config_build(intcur->cur_essid);
 		      config_set_globals(intcur);
 		      intcur->next = NULL;
@@ -612,7 +462,7 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
 
       // Here, we need to check, and add interfaces that are included in the
       // allow, but were not found already. 
-      add_other_allowed_interfaces(&int_list, *dcfg);
+      add_other_allowed_interfaces(&int_list);
 
     } else {
 
@@ -628,7 +478,7 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
       memset(int_list, 0, sizeof(struct interface_data));
 
       // Start by setting up the structure, and assigning the interface.
-      if (init_interface_struct(int_list, device, *dcfg) != XENONE)
+      if (init_interface_struct(int_list, device) != XENONE)
 	{
 	  debug_printf(DEBUG_NORMAL, "Couldn't init interface struct for device %s!  Cannot continue!\n", device);
 	  exit(1);
@@ -638,18 +488,16 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
       if (cardif_init(int_list) < 0) return XENOTINT;
       debug_printf(DEBUG_NORMAL, "Interface initalized!\n");
   
-      // If we don't know what kind of interface this is....
-      if (int_list->isWireless == -1)
+      // Check to see if this interface is wireless.
+      if (cardif_int_is_wireless(int_list->intName) == TRUE)
 	{
-	  // Check to see if this interface is wireless.
-	  if (cardif_int_is_wireless(int_list->intName) == TRUE)
-	    {
-	      int_list->isWireless = TRUE;
-	      cardif_check_dest(int_list);
-	    } else {
-	      int_list->isWireless = FALSE;
-	    }
+	  SET_FLAG(int_list->flags, IS_WIRELESS);
+	  cardif_check_dest(int_list);
+	  cardif_start_wireless_scan(int_list);
+	} else {
+	  UNSET_FLAG(int_list->flags, IS_WIRELESS);
 	}
+
       int_list->userdata = config_build(int_list->cur_essid);
       config_set_globals(int_list);
       int_list->next = NULL;
@@ -660,6 +508,14 @@ int global_init(int new_debug, int xdaemon, char *device, char *config,
   return XENONE;
 }
 
+void global_sigseg()
+{
+    fprintf(stderr, "[FATAL] Coredump!!!\n");
+    fflush(stderr); fflush(stdout);
+    delete_pidfile();
+    exit(-1);
+}
+
 /****************************************
  *
  * Clean up any values that we have set up for use by the supplicant.  This
@@ -702,6 +558,8 @@ void global_deinit()
       intcur=destroy_interface_struct(intcur);
     }
 
+  event_core_cleanup();
+
   config_destroy();
 
   logfile_cleanup();
@@ -713,6 +571,24 @@ void global_deinit()
   exit(0);
 }
 
+/****************************************
+ *
+ * Reap dead children
+ *
+ ****************************************/
+
+void global_child(int signum)
+{
+	int pid,status;
+	while (1) {
+		pid = waitpid (WAIT_ANY, &status, WNOHANG);
+		if(pid <= 0){
+			break;
+		}
+	}
+}
+
+
 
 /****************************************
  *
@@ -726,7 +602,20 @@ void usage(char *prog)
 	       "[-c config file] "
 	       "[-i device] "
 	       "[-d debug_level] "
+	       "[-f] "
 	       "\n", prog);
+
+  debug_printf(DEBUG_NORMAL, " <debug_level> can be any of : \n");
+  debug_printf(DEBUG_NORMAL, "\t0..7 : Old style debug flags.\n");
+  debug_printf(DEBUG_NORMAL, "\tA : Enable ALL debug flags.\n");
+  debug_printf(DEBUG_NORMAL, "\tc : Enable CONFIG debug flag.\n");
+  debug_printf(DEBUG_NORMAL, "\ts : Enable STATE debug flag.\n");
+  debug_printf(DEBUG_NORMAL, "\ta : Enable AUTHTYPES debug flag.\n");
+  debug_printf(DEBUG_NORMAL, "\ti : Enable INT debug flag.\n");
+  debug_printf(DEBUG_NORMAL, "\tn : Enable SNMP debug flag.\n");
+  debug_printf(DEBUG_NORMAL, "\te : Enable EVERYTHING debug flag.\n");
+  debug_printf(DEBUG_NORMAL, "\tx : Enable EXCESSIVE debug flag.\n");
+  debug_printf(DEBUG_NORMAL, "\nValues [0..7] cannot be combined with character attributes!\n");
 }
 
 /***************************************
@@ -738,12 +627,10 @@ void usage(char *prog)
  ***************************************/
 int main(int argc, char *argv[])
 {
-  int op, pid, ssid, mac;
+  int op, pid;
   char *theOpts = "c:i:d:Wf";
-  char *dstAddr = NULL, *config = NULL, *device = NULL, *netid = NULL;
-  char *username = NULL, *password = NULL, *auth_method = NULL;
-  int xdaemon = 1, new_debug, firsttime = 0, framewaiting = 0;
-  struct daemon_conf *dconf = NULL;
+  char *config = NULL, *device = NULL;
+  int xdaemon = 1, new_debug;
   struct interface_data *intcur = NULL;
 
   if (is_other_xsup_running() == TRUE)
@@ -774,7 +661,12 @@ int main(int argc, char *argv[])
 
 	case 'd':
 	  // Set the debug level.
-	  new_debug = atoi(optarg);
+	  if ((atoi(optarg) == 0) && (optarg[0] != '0'))
+	    {
+	      debug_alpha_set_flags(optarg);
+	    } else {
+	      debug_set_flags(atoi(optarg));
+	    }
 	  break;
 
 	case 'W':
@@ -796,18 +688,9 @@ int main(int argc, char *argv[])
 	}
     }
 
-  if ((xdaemon == 1) && ((dstAddr != NULL) || 
-			(device != NULL) || (netid != NULL) || 
-			(username != NULL) || (password != NULL) ||
-			(auth_method != NULL)))
-    {
-      printf("You cannot override configuration file options when running\n");
-      printf("in daemon mode!  Options will be ignored!\n");
-    }
-
   if (xdaemon == 1)
     {
-      printf("Starting XSupplicant v. %s!\n",XVERSION);
+      printf("Starting XSupplicant v. %s!\n",VERSION);
       // We should fork, and let the parent die.
       pid = fork();
       
@@ -821,7 +704,7 @@ int main(int argc, char *argv[])
     }
 
   // We have our options set, to initalize, then go in to our event loop.
-  if (global_init(new_debug, xdaemon, device, config, &dconf) != 0)
+  if (global_init(xdaemon, device, config) != 0)
     {
       printf("Couldn't initalize!!!!\n");
       exit(255);
@@ -846,12 +729,13 @@ int main(int argc, char *argv[])
   signal(SIGINT, global_deinit);
   signal(SIGQUIT, global_deinit);
   signal(SIGKILL, global_deinit);
+  signal(SIGSEGV, global_sigseg);
 
   // Create our pidfile.
   // int_list->sockInt isn't really what we want to pass in here.  We *Really*
   // want to pass in the socket # that the client should talk to us on.  This
   // will allow us to be socket agnostic.
-  if(!create_pidfile(int_list->sockInt))
+  if(!create_pidfile(0))
     {
       printf("Couldn't create pid file!\n");
     }
@@ -860,65 +744,28 @@ int main(int argc, char *argv[])
   signal(SIGALRM, alrmclock);
   alarm(1);
 
-  while (1!=0)
+  // we handle our dead children
+  signal(SIGCHLD, global_child);
+
+  // We just started, so execute our startup command.
+  intcur = int_list;
+  while (intcur != NULL)
+    {
+      cmd_handler_exec(intcur, config_get_startup_cmd());
+      intcur = intcur->next;
+    }
+
+  while (1)
     {
-      struct timeval tv;
-      fd_set rfds;
-      int maxfd = 0;
       intcur = int_list;
 
       xsup_ipc_process(intcur);
 
-      FD_ZERO(&rfds);
-      while (intcur != NULL)
-	{
-	  debug_printf(DEBUG_EVERYTHING, "Processing interface %s...\n",intcur->intName);
-
-	  if (firsttime == 0)
-	    {
-	      // Execute our startup command before we do anything else.
-	      cmd_handler_exec(intcur, config_get_startup_cmd());
-	    }
-
-	  // First, check our destination address, in case we have hopped APs.
-	  mac = cardif_check_dest(intcur);
-	  ssid = cardif_check_ssid(intcur);
-
-	  if ((mac == TRUE) && (ssid != XNEWESSID))
-	    {
-	      debug_printf(DEBUG_CONFIG, "MAC address changed!  Updating config!\n");
-	      intcur->userdata = config_build(intcur->cur_essid);
-	    }
-
-	  if (ssid == XNEWESSID)
-	    {
-	      // Our ESSID changed, we need to clear our config, so that
-	      // we generate a new one on the way through the statemachine.
-	      eap_clear_active_method(intcur);
-
-	      // We also need to check to make sure we are pointing
-	      // to the correct configuration information.
-	      intcur->userdata = config_build(intcur->cur_essid);
-	    }
-	  eapol_execute(intcur);
-	  FD_SET(intcur->sockInt, &rfds);
-	  if (intcur->sockInt > maxfd)
-	    maxfd = intcur->sockInt;
-	  
-	  // If we have a frame available on any interface, we won't
-	  // sleep.  Instead we will keep looping through to keep things
-	  // moving as fast as possible.
-	  if (frameavail(intcur) == TRUE) framewaiting = 1;
-	  intcur = intcur->next;
-	}
-      firsttime = 1;
-      if (framewaiting == 0)
-	{
-	  tv.tv_sec = 0;
-	  tv.tv_usec = 500000;
-	  select(maxfd + 1, &rfds, NULL, NULL, &tv);
-	}
-      framewaiting = 0;
+      // There is a chance that we will need to modify the interface
+      // structure inside of the event core.  (Examples include adding
+      // an interface, or removing one.)  Since this is OS specific, we
+      // need to call it in the event routines.
+      event_core(&intcur);
     }
 
   return XENONE;
