@@ -58,62 +58,16 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: xsup_ipc.c,v 1.14 2004/04/05 17:19:16 chessing Exp $
- * $Date: 2004/04/05 17:19:16 $
+ * $Id: xsup_ipc.c,v 1.16 2004/07/15 04:15:35 chessing Exp $
+ * $Date: 2004/07/15 04:15:35 $
  * $Log: xsup_ipc.c,v $
- * Revision 1.14  2004/04/05 17:19:16  chessing
+ * Revision 1.16  2004/07/15 04:15:35  chessing
  *
- * Added additional checks against pointers to try to help prevent segfaults.  (This still needs to be completed.)  Fixed a problem with PEAP where a NULL input packet would result in a huge unencrypted packet, and a segfault.  (This was triggered when using one of the gui password tools.  When the password was in the config file, it wouldn't be triggered.)
+ * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
  *
- * Revision 1.13  2004/03/29 21:57:14  chessing
+ * Revision 1.15  2004/06/15 03:22:29  chessing
  *
- * Changed the socket number we use for communication with the daemon from 10240 (which seems like a bad choice) to 26798 (which seems a little more random ;).  Also changed our debug code so that it doesn't output to the console when we are running in daemon mode.  The only way to get debug info while in daemon mode is to set a log file!!!
- *
- * Revision 1.12  2004/03/27 02:20:07  chessing
- *
- * Fixed a problem where the IPC socket wasn't getting deallocated correctly, and would keep xsupplicant from running a second time.  Added the needed hooks to make PEAP-GTC work.  (Not tested yet.)
- *
- * Revision 1.11  2004/03/27 01:40:45  chessing
- *
- * Lots of small updates to free memory that wasn't getting freed, add some additional debug output, and fix a couple of memory leaks.
- *
- * Revision 1.10  2004/03/26 21:34:51  chessing
- * Fixed problem with interface being down on startup causing xsupplicant to not read the proper configuration information when the interface is brought up.  Added/fixed code to rebuild userdata piece of structure when the essid changes.  Added code to avoid setting a key on an interface if the interface doesn't already have encryption enabled.  Added a little bit of debugging code to help find a solution to an IPC socket problem.
- *
- * Revision 1.9  2004/03/17 21:21:40  chessing
- *
- * Hopefully xsup_set_pwd is in the right place now. ;)  Added the functions needed for xsupplicant to request a password from a GUI client.  (Still needs to be tested.)  Updated TTLS and PEAP to support password prompting.  Fixed up curState change in statemachine.c, so it doesn't print [ALL] in front of the current state.
- *
- * Revision 1.8  2004/02/11 03:31:07  npetroni
- * simple check to avoid segfault in ipc cleanup
- *
- * Revision 1.7  2004/01/18 06:31:19  chessing
- *
- * A few fixes here and there.  Added support in EAP-TLS to wait for a password to be entered from a "GUI" interface.  Added a small CLI utility to pass the password in to the daemon. (In gui_tools/cli)  Made needed IPC updates/changes to support passing in of a generic password to be used.
- *
- * Revision 1.6  2004/01/14 05:44:48  chessing
- *
- * Added pid file support. (Very basic for now, needs to be improved a little.)  Attempted to add setup of global variables. (Need to figure out why it is segfaulting.)  Added more groundwork for IPC.
- *
- * Revision 1.5  2003/12/23 04:57:10  chessing
- *
- * IPC additions, GUI client routines.
- *
- * Revision 1.4  2003/12/19 06:29:56  chessing
- *
- * New code to determine if an interface is wireless or not.  Lots of IPC updates.
- *
- * Revision 1.3  2003/12/18 02:09:45  chessing
- *
- * Some small fixes, and working IPC code to get interface state.
- *
- * Revision 1.2  2003/12/14 06:11:03  chessing
- *
- * Fixed some stuff with SIM in relation to the new config structures.  Cleaned out CR/LF from LEAP source files.  Added user certificate support to TTLS and PEAP. Some additions to the IPC code. (Not tested yet.)
- *
- * Revision 1.1  2003/12/07 06:20:19  chessing
- *
- * Changes to deal with new config file style.  Beginning of IPC code.
+ * XSupplicant Release 1.0
  *
  *
  *******************************************************************/
@@ -132,6 +86,7 @@
 #include "xsup_err.h"
 #include "ipc_callout.h"
 
+static struct ipc_struct *ipc = NULL;
 
 /******************************************************************
  *
@@ -139,21 +94,13 @@
  * Also, set up any structures that may be needed.
  *
  ******************************************************************/
-int xsup_ipc_init(struct interface_data *startint)
+int xsup_ipc_init()
 {
   int sockErr = 0;
   int sockOpts;
   int sockdesc;
   char *error = NULL;
   struct sockaddr_in sa;
-  struct ipc_struct *ipc;
-  struct interface_data *cur;
-
-  if (!startint)
-    {
-      debug_printf(DEBUG_NORMAL, "Invalid interface passed in to xsup_ipc_init()!\n");
-      return XEMALLOC;
-    }
 
   sa.sin_family = AF_INET;
   sa.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
@@ -199,13 +146,6 @@ int xsup_ipc_init(struct interface_data *startint)
   ipc->sockid = sockdesc;
   ipc->reged = NULL;
 
-  cur = startint;
-  while (cur != NULL)
-    {
-      cur->ipc = ipc;
-      cur = cur->next;
-    }
-
   return XENONE;
 }
 
@@ -302,24 +242,23 @@ void xsup_ipc_get_message(int sockdesc, char *outbuf, int *outsize,
  * Send a message to all registered clients.
  *
  ***********************************************************/
-int xsup_ipc_send_all_registered(struct interface_data *thisint, char *message,
-				 int msglen)
+int xsup_ipc_send_all_registered(char *message, int msglen)
 {
   struct registered_clients *cur;
 
-  if ((!thisint) || (!thisint->ipc) || (!message))
+  if (!message)
     {
       debug_printf(DEBUG_NORMAL, "Invalid data passed in to xsup_ipc_send_all_registered()!\n");
       return XEMALLOC;
     }
 
-  cur = thisint->ipc->reged;
+  cur = ipc->reged;
 
   while (cur != NULL)
     {
       // Send a message to the client that is currently registered.
-      xsup_ipc_send_message(thisint->ipc->sockid, cur->addr, 
-			    sizeof(struct sockaddr), message, msglen);
+      xsup_ipc_send_message(ipc->sockid, cur->addr, sizeof(struct sockaddr), 
+			    message, msglen);
 
       cur = cur->next;
     }
@@ -383,7 +322,7 @@ struct interface_data *xsup_ipc_select_int(char *interface,
   ipc_callout_send_error(thisint, &bufptr, (char *)&outbuf[0], 1500, 
 			 (char *)&retstr);
 
-  xsup_ipc_send_message(thisint->ipc->sockid, tohost, tohost_len, &outbuf[0], 
+  xsup_ipc_send_message(ipc->sockid, tohost, tohost_len, &outbuf[0], 
 			bufptr);
   return NULL;
 }
@@ -408,7 +347,7 @@ void xsup_ipc_process(struct interface_data *thisint)
       return;
     }
 
-  if (!thisint->ipc)
+  if (!ipc)
     {
       debug_printf(DEBUG_NORMAL, "No ipc information available!\n");
       return;
@@ -418,14 +357,12 @@ void xsup_ipc_process(struct interface_data *thisint)
   resbufptr = 0;
   numcmds = 0;
 
-  //  debug_printf(DEBUG_NORMAL, "Socket descriptor is #%d\n",thisint->ipc->sockid);
-
   rethead = (struct ipc_header *)&result_buf[0];
   bzero(&result_buf[0], 1520);
 
   inhead = (struct ipc_header *)&buffer[0];
 
-  xsup_ipc_get_message(thisint->ipc->sockid, (char *)&buffer, &bufsize, 
+  xsup_ipc_get_message(ipc->sockid, (char *)&buffer, &bufsize, 
 		       (char *)&interface, &mysock, &salen);
 
   // If our buffer is 0, then we don't have anything to process.
@@ -479,7 +416,7 @@ void xsup_ipc_process(struct interface_data *thisint)
 	  if (inhead->getset == IPC_SET)
 	    {
 	      debug_printf(DEBUG_EVERYTHING, "Registering client!\n");
-	      ipc_callout_reg_client(workint, &bufptr, (char *)&buffer, 
+	      ipc_callout_reg_client(ipc, &bufptr, (char *)&buffer, 
 				     bufsize, (char *)&result_buf, &resbufptr,
 				     &mysock);
 	      numcmds++;
@@ -532,7 +469,7 @@ void xsup_ipc_process(struct interface_data *thisint)
       debug_printf(DEBUG_EVERYTHING, "Sending IPC response!\n");
       rethead->numcmds = numcmds;
 
-      xsup_ipc_send_message(thisint->ipc->sockid, &mysock, salen, 
+      xsup_ipc_send_message(ipc->sockid, &mysock, salen, 
 			    (char *)&result_buf, resbufptr);
     }
 }
@@ -542,30 +479,29 @@ void xsup_ipc_process(struct interface_data *thisint)
  * Clean up any structures used, and close out the communication socket.
  *
  ***********************************************************/
-void xsup_ipc_cleanup(struct interface_data *startint)
+void xsup_ipc_cleanup()
 {
   struct registered_clients *cur, *next;
   char *error;
 
-  if (!startint || !startint->ipc)
+  if (!ipc)
     return;
   
-  cur = startint->ipc->reged;
+  cur = ipc->reged;
   while (cur != NULL)
     {
       next = cur->next;
       free(cur);
       cur = next;
     }
-  //  free(startint->ipc);
 
   debug_printf(DEBUG_EVERYTHING, "Shutting down IPC socket!\n");
-  debug_printf(DEBUG_CONFIG, "Closing socket descriptor #%d\n", startint->ipc->sockid);
-  if (close(startint->ipc->sockid) < 0)
+  debug_printf(DEBUG_CONFIG, "Closing socket descriptor #%d\n", ipc->sockid);
+  if (close(ipc->sockid) < 0)
     {
       error = strerror(errno);
       debug_printf(DEBUG_NORMAL, "Error closing socket!  (Error : %s)\n",
 		   error);
     }
-  free(startint->ipc);
+  free(ipc);
 }
